import { createServer } from "http";
import { parse } from "url";
import { WebSocketServer } from "ws";

process.env["NODE_TLS_REJECT_UNAUTHORIZED"] = "0";

import { Configuration } from "./configuration.js";
import { OsmondAuth } from "./osmond.js";
import { DeviceManager } from "./deviceManager.js";

let config = new Configuration("readers.json");
let deviceManager = new DeviceManager();
config.load(deviceManager);

const server = createServer();
const wss = new WebSocketServer({ noServer: true });

//on ws connection
wss.on("connection", function connection(ws, request) {
  let deviceName = request.url.substring(1);
  let communicationChannel = deviceManager.getCommunicationChannel(deviceName);
  if (communicationChannel == null || communicationChannel.channel == null) {
    wss.emit("close", ws, request);
    ws.close();
    return;
  }
  communicationChannel.channel.on("open", () => {
    console.log("deviceChannel opened to OSMOND device:");
  });

  communicationChannel.channel.on("message", function message(data) {
    console.log("message from OSMOND device:" + data);
    ws.send(data.toString("utf8"));
    console.log("message transmitted to Vue application");
  });

  communicationChannel.channel.on("close", () => {
    ws.close();
    console.log(
      "deviceChannel closed, ws connection to Vue application closed."
    );
  });

  communicationChannel.channel.on("error", (error) => {
    console.log("deviceChannel webscoket error: " + error);
  });

  ws.on("error", function message(data) {
    console.log("ws error");
  });

  ws.on("message", function message(data) {
    console.log(
      "Received message" + data + " from user:" + request.socket.remoteAddress
    );
    communicationChannel.channel.send(data);
    console.log(`Message transmitted to deviceChannel (OSMOND device)`);
  });
 
  ws.on("close", () => {
    console.log("Client channel closed.");
    communicationChannel.channel.close();
  });
});

wss.on("error", function message(data) {
  console.log("wss error");
});
//request handlers

server.on("request", function request(request, response) {
  const { pathname } = parse(request.url);
  if (pathname.startsWith("/login")) {
    console.log("http login");
    const deviceName = request.url.substring(7);
    let connectionParams = deviceManager.getAuthenticationParams(deviceName);
    if (connectionParams === null) {
      response.statusCode = 404;
      response.setHeader("Access-Control-Allow-Origin", "*");
      response.end("");
      return;
    }
    let osmond = new OsmondAuth(
      connectionParams.hostname,
      connectionParams.username,
      connectionParams.password
    );
    const api_token = osmond.httpLogin();
    let deviceChannel = osmond.wsLogin(api_token);

    deviceManager.setCommunicationChannel(deviceName, deviceChannel);
    response.setHeader("Access-Control-Allow-Origin", "*");
    response.end("");
  } else if (pathname.startsWith("/logout")) {
    console.log("http logout");
    const deviceName = request.url.substring(8);
    let authParams = deviceManager.getAuthenticationParams(deviceName);
    let commParams = deviceManager.getCommunicationChannel(deviceName);
    if (authParams === null || commParams === null) {
      response.statusCode = 404;
      response.end("");
    }
    const osmond = new OsmondAuth(
      authParams.hostname,
      authParams.username,
      authParams.password
    );
    osmond.httpLogout(commParams.token);
    response.setHeader("Access-Control-Allow-Origin", "*");
    response.statusCode = 200;
    response.end("");
  } else if (pathname.startsWith("/devicelist")) {
    let devices = deviceManager.getDeviceNames();
    let responseText = JSON.stringify(devices);
    response.setHeader("Access-Control-Allow-Origin", "*");
    response.setHeader("Content-Type", "application/json");
    response.statusCode = 200;
    console.log("http devlist response:" + responseText);
    response.end(responseText);
  }
});

server.on("upgrade", function upgrade(request, socket, head) {
  wss.handleUpgrade(request, socket, head, function done(ws) {
    wss.emit("connection", ws, request);
  });
});

server.listen(8072);
