/*******************************************************************************
 * GX sample program
 *
 * Zoom image example.
 ******************************************************************************/

#include "gxsd.h"
#include "gxerror.h"
#include "gximage.h"
#include "gxsdldr.c"

/*******************************************************************************
 * If an error occurred it displays the error and terminates the program
 ******************************************************************************/
void term_if_err(int st) {
	int err_code;
	wchar_t err_str[256];
	err_str[(sizeof(err_str)/sizeof(wchar_t))-1] = 0;

	if(st) return;		/* No error occurred */

	/* Displays error code and description */
	gx_geterror(&err_code, err_str, (sizeof(err_str)/sizeof(wchar_t))-1);
	fprintf(stderr, "GX error occurred: %i %ls\n", err_code, err_str);

	/* Terminates the program */
	exit(1);
}

/*******************************************************************************
 * Main function
 ******************************************************************************/
int main(void) {
	int st;
	struct gxHANDLE imghandle;
	struct gxHANDLE linearhndl;
	struct gxHANDLE nearesthndl;
	struct gxIMAGE *image;
	struct gxIMAGE *linearzoomed;
	struct gxIMAGE *nearestzoomed;
	
	/* Creates a new image handler with default properties */
	st = gx_openmodule(&imghandle, L"gximage", L"default");
	term_if_err(st);

	/* Creates an image handler for linear zoom */
	st = gx_openmodule(&linearhndl, L"gximage", L"default");
	term_if_err(st);
	/* Updates the zoom mode */
	st = gx_setmodulepropertyint(linearhndl, L"zoom_mode", GX_ZOOM_LINEAR);
	term_if_err(st);

	/* Creates an image handler for nearest zoom */
	st = gx_openmodule(&nearesthndl, L"gximage", L"default");
	term_if_err(st);
	/* Updates the zoom mode */
	st = gx_setmodulepropertyint(nearesthndl, L"zoom_mode", GX_ZOOM_NEAREST);
	term_if_err(st);
	
	/* Creates a new image structure */
	st = gx_allocimage(imghandle, &image);
	term_if_err(st);
	/* Loads the sample image */
	st = gx_loadimage(imghandle, image, L"../../../data/mushroom.jpg", GX_UNDEF);
	term_if_err(st);
	
	/* Creates an image object for linear zoom operations */
	st = gx_allocimage(linearhndl, &linearzoomed);
	term_if_err(st);
	/* Creates an image object for nearest zoom operations */
	st = gx_allocimage(nearesthndl, &nearestzoomed);
	term_if_err(st);
	
	/* Diminutives and saves the image */
	st = gx_zoomimage(linearhndl, linearzoomed, image->xsize/4, image->ysize/4, 0,
					image, 0, 0, 0, 0);
	term_if_err(st);
	st = gx_saveimage(linearhndl, linearzoomed, L"test_lzoom1.jpg", GX_JPEG);
	term_if_err(st);
	
	st = gx_zoomimage(nearesthndl, nearestzoomed, image->xsize/4, image->ysize/4, 0,
					image, 0, 0, 0, 0);
	term_if_err(st);
	st = gx_saveimage(nearesthndl, nearestzoomed, L"test_nzoom1.jpg", GX_JPEG);
	term_if_err(st);
	
	/* Magnifies and saves the image */
	st = gx_zoomimage(linearhndl, linearzoomed, image->xsize*4, image->ysize*4, 0,
					image, 0, 0, 0, 0);
	term_if_err(st);
	st = gx_saveimage(linearhndl, linearzoomed, L"test_lzoom2.jpg", GX_JPEG);
	term_if_err(st);

	st = gx_zoomimage(nearesthndl, nearestzoomed, image->xsize*4, image->ysize*4, 0,
					image, 0, 0, 0, 0);
	term_if_err(st);
	st = gx_saveimage(nearesthndl, nearestzoomed, L"test_nzoom2.jpg", GX_JPEG);
	term_if_err(st);
	
	/* Zooms an area (58x35 - 316x191) and saves it */
	st = gx_zoomimage(linearhndl, linearzoomed, 258, 156, 0,
					image, 256*58, 256*35, 256*(58+258), 256*(35+156));
	term_if_err(st);
	st = gx_saveimage(linearhndl, linearzoomed, L"test_lzoom3.jpg", GX_JPEG);
	term_if_err(st);

	st = gx_zoomimage(nearesthndl, nearestzoomed, 258, 156, 0,
					image, 256*58, 256*35, 256*(58+258), 256*(35+156));
	term_if_err(st);
	st = gx_saveimage(nearesthndl, nearestzoomed, L"test_nzoom3.jpg", GX_JPEG);
	term_if_err(st);

	/* Frees the linearzoomed structure */
	st = gx_unrefimage(linearhndl, linearzoomed);
	term_if_err(st);
	/* Frees the nearestzoomed structure */
	st = gx_unrefimage(nearesthndl, nearestzoomed);
	term_if_err(st);
	/* Frees the image structure */
	st = gx_unrefimage(imghandle, image);
	term_if_err(st);

	/* Closes the opened image modules */
	st = gx_closehandle(&linearhndl);
	term_if_err(st);
	st = gx_closehandle(&nearesthndl);
	term_if_err(st);
	st = gx_closehandle(&imghandle);
	term_if_err(st);

	return 0;
}
