/*******************************************************************************
 * GX sample program
 *
 * Image file formats example.
 ******************************************************************************/

#include "gxsd.h"
#include "gxerror.h"
#include "gximage.h"
#include "gxsdldr.c"

#ifdef WIN32
	#define snprintf _snprintf
#endif

/*******************************************************************************
 * If an error occurred it displays the error and terminates the program
 ******************************************************************************/
void term_if_err(int st) {
	int err_code;
	wchar_t err_str[256];
	err_str[(sizeof(err_str)/sizeof(wchar_t))-1] = 0;

	if(st) return;		/* No error occurred */

	/* Displays error code and description */
	gx_geterror(&err_code, err_str, (sizeof(err_str)/sizeof(wchar_t))-1);
	fprintf(stderr, "GX error occurred: %i %ls\n", err_code, err_str);

	/* Terminates the program */
	exit(1);
}

/*******************************************************************************
 * Content of an RGB pixel with alpha component
 ******************************************************************************/
#pragma pack(1)
typedef struct {
	unsigned char red;
	unsigned char green;
	unsigned char blue;
	unsigned char alpha;
} myRGB;
#pragma pack()

/*******************************************************************************
 * Type for format storing
 ******************************************************************************/
typedef struct {
	const char *name;
	int			code;
} FORMAT;

/*******************************************************************************
 * Saves an image with different file formats
 ******************************************************************************/
void save_images(struct gxHANDLE handle, struct gxIMAGE *image, const char *nstr) {
	char name[64];
	int ix = 0, st;
	const FORMAT formats[] = {
		{	"bmp",		GX_BMP			},
		{	"png",		GX_PNG			},
		{	"raw",		GX_RAW			},
		{	"jpg",		GX_JPEG			},
		{	"jp2",		GX_JPEG2K_JP2	},
		{	"jpc",		GX_JPEG2K_JPC	},
		
		{	(const char *)0, 0			}
	};

	name[sizeof(name)-1] = 0;
	
	/* Saves the image */
	while(formats[ix].name) {
		snprintf(name, sizeof(name)-1, "test_%s.%s", nstr, formats[ix].name);
		printf("%s ", name); fflush(stdout);

		st = gx_saveimagea(handle, image, name, formats[ix].code);
		term_if_err(st);
		
		printf("saved.\n");
		
		ix++;
	}
}

/*******************************************************************************
 * Main function
 ******************************************************************************/
int main(void) {
	int st, ix, i, j;
	struct gxHANDLE handle;
	struct gxIMAGE *image;
	myRGB *ptr;
	struct gxIMAGE *temp;

	const FORMAT formats[] = {
		{	"rgb",		GX_RGB		},
		{	"bgr",		GX_BGR		},
		{	"rgba", 	GX_RGBA		},
		{	"bgra", 	GX_BGRA		},
		{	"rgb12",	GX_RGB12	},
		{	"bgr12",	GX_BGR12	},
		{	"rgb555",	GX_RGB555	},
		{	"bgr555",	GX_BGR555	},
		{	"rgb565",	GX_RGB565	},
		{	"bgr565",	GX_BGR565	},
		{	"gray", 	GX_GRAY		},
		{	"gray12", 	GX_GRAY12	},
		{	"yuv422", 	GX_YUV422	},
		
		{	(const char *)0, 0		}
	};
	
	/* Creates a new image handle with default properties */
	st = gx_openmodule(&handle, L"gximage", L"default");
	term_if_err(st);
	
	/* Creates an image structure */
	st = gx_allocimage(handle, &image);
	term_if_err(st);
	
	/* Creates the 512x512 RGBA image */
	st = gx_createimage(handle, image, GX_RGBA, 512, 512, 0);
	term_if_err(st);
	
	for(j = 0; j < image->ysize; j++) {
		ptr = (myRGB *)gx_getpixelptr(handle, image, 0, j);
		for(i = 0; i < image->xsize; i++) {
			ptr[i].red = j;
			ptr[i].green = i;
			ptr[i].blue = j+i;
			ptr[i].alpha = 0x80;
		}
	}
	
	/* Saves images with various pixel format */
	ix = 0;
	st = gx_allocimage(handle, &temp);
	term_if_err(st);
	
	while(formats[ix].name) {
		/* Converts the image to a pixel format */
		printf("Converting to %s.. ", formats[ix].name); fflush(stdout);
		
		st = gx_convertimage(handle, temp, image, formats[ix].code,
					image->xsize, image->ysize, 0);
		term_if_err(st);
		
		printf("Done\n");
		
		
		/* Saves the image to any file formats */
		save_images(handle, temp, formats[ix].name);
		
		ix++;
	}
	
	/* Deletes images */
	st = gx_unrefimage(handle, temp);
	term_if_err(st);
	st = gx_unrefimage(handle, image);
	term_if_err(st);
	
	/* Closes the opened module */
	st = gx_closehandle(&handle);
	term_if_err(st);
	
	return 0;
}
