/******************************************************************************/
/* Library functions for the pr sample programs.                              */
/******************************************************************************/
#ifndef __LIB__INCL
#define __LIB__INCL
/******************************************************************************/
#if defined WIN32
	#include <conio.h>
	#include <io.h>
#elif defined LINUX
	#include <unistd.h>
	#include <dirent.h>
	#include <sys/select.h>
#else
	#error OS or compiler error in __lib__.h
#endif
/******************************************************************************/
#include <sys/timeb.h>
#include <sys/stat.h>
#define inline	static
#define bool	int
#define true	1
#define false	0

/******************************************************************************/
/* Helper functions.                                                          */
/******************************************************************************/

	int _loop_count = 0;
	int _err_count = 0;
	struct timeb _start_, _stop_;
	struct timeb _process_start_, _process_stop_;
	int _process_avg = 0;
	char _str[80] = "";

	/**************************************************************************/
	/* Detects keyboard events.                                               */
	/**************************************************************************/
	inline bool lib_kbhit(void) {
		#ifdef _MSC_VER
			return _kbhit() != 0;
		#elif defined WIN32
			return kbhit() != 0;
		#elif defined LINUX
			fd_set rfds;
			FD_ZERO(&rfds);
			FD_SET(STDIN_FILENO, &rfds);

			struct timeval tv;
			tv.tv_sec = 0;
			tv.tv_usec = 0;

			return select(STDIN_FILENO+1, &rfds, NULL, NULL, &tv) > 0;
		#endif
	}

	/**************************************************************************/
	/* Displays the string and flushes the output stream.                     */
	/**************************************************************************/
	inline void lib_write(const char *fmt, ...) {
		va_list args;
		va_start(args, fmt);
		vprintf(fmt, args);
		va_end(args);
		fflush(stdout);
	}

	/**************************************************************************/
	/* Displays the string.                                                   */
	/**************************************************************************/
	inline void lib_write_line(const char *fmt, ...) {
		va_list args;
		va_start(args, fmt);
		vprintf(fmt, args);
		va_end(args);
		printf("\n");
	}

	/**************************************************************************/
	/* Reads a string from the standard input.                                */
	/**************************************************************************/
	inline const char * lib_read_line(const char *prompt) {
		if (prompt) lib_write(prompt);
		#if _MSC_VER >= 1400
			scanf_s("%s", _str, 80);
		#else
			scanf("%s", _str);
		#endif
		return _str;
	}

	/**************************************************************************/
	/* Displays the description and the last GX error.                        */
	/**************************************************************************/
	inline int lib_displ_err(void) {
		int err_code;
		char err_str[256];
		memset(err_str, 0, sizeof(err_str));
		gx_geterrora(&err_code, err_str, sizeof(err_str)-1);
		if (err_code != 0) {
			fprintf(stderr, "Error (%#08x): %s\n", err_code, err_str);
			_err_count++;
		}
		return err_code;
	}

	/**************************************************************************/
	/* Displays the error message and exits the program.                      */
	/**************************************************************************/
	inline void lib_error(const char *str) {
		_err_count++;
		if (str) fprintf(stderr, "%s\n", str);
	}

	/**************************************************************************/
	/* Displays the start message and starts timer.                           */
	/**************************************************************************/
	inline void lib_function_start(const char *fmt, ...) {
		va_list args;
		va_start(args, fmt);
		vprintf(fmt, args);
		printf("... ");
		fflush(stdout);
		ftime(&_start_);
	}

	/**************************************************************************/
	/* Displays the end message and returns the time elapsed in ms.           */
	/**************************************************************************/
	inline int lib_function_end(void) {
		int t;
		ftime(&_stop_);
		t = _stop_.millitm + (int)(_stop_.time - _start_.time) * 1000 - _start_.millitm;
		lib_write_line("Done. (%i ms)", t);
		return t;
	}

	/**************************************************************************/
	/* Displays the process start message and starts the process timer.       */
	/**************************************************************************/
	inline void lib_process_start(const char *str) {
		lib_write_line("\n*** %s [%i] ***", str, ++_loop_count);
		ftime(&_process_start_);
	}

	/**************************************************************************/
	/* Displays the end message and returns the time elapsed in ms.           */
	/**************************************************************************/
	inline int lib_process_end(void) {
		int _process_time;
		ftime(&_process_stop_);
		_process_time = _process_stop_.millitm + (int)(_process_stop_.time - _process_start_.time) * 1000 - _process_start_.millitm;
		_process_avg = ((_loop_count-1) * _process_avg + _process_time) / _loop_count;
		lib_write_line("*** Process time: %i ms, avg: %i", _process_time, _process_avg);
		return _process_time;
	}

	/**************************************************************************/
	/* Displays overall statistics.                                           */
	/**************************************************************************/
	int lib_print_stat(void) {
		printf("\n*** Statistics ***\n\n");
		printf("Number of loops : %i\n", _loop_count);
		printf("Number of errors: %i\n", _err_count);
		printf("Process average : %i ms\n\n", _process_avg);
		#ifdef WIN32
			system("pause");
		#endif
		return _loop_count;
	}

	/**************************************************************************/
	/* Saves binary data in file.                                             */
	/**************************************************************************/
	inline bool lib_save(const char *fn, void *data, int size) {
		FILE *fp;
		#if _MSC_VER >= 1400
			if (fopen_s(&fp, fn, "wb")) fp = NULL;
		#else
			fp = fopen(fn, "wb");
		#endif
		if (!fp) return false;
		if (fwrite(data, 1, size, fp) != (size_t)size) {
			fclose(fp);
			return false;
		}
		fclose(fp);
		return true;
	}

	/**************************************************************************/
	/* Waits for a time specified in miliseconds.                             */
	/**************************************************************************/
	inline void lib_wait(int ms) {
		#if defined WIN32
			Sleep(ms);
		#elif defined LINUX
			sleep(ms/1000);
			usleep((ms%1000)*1000);
		#endif
	}

	/**************************************************************************/
	/* Waits for n seconds.                                                   */
	/**************************************************************************/
	inline void lib_wait_for_sec(int sec) {
		int i;
		for (i=sec; i>0; i--) {
			if (lib_kbhit()) break;
			lib_write("\rWaiting for %i sec...  ", i);
			lib_wait(1000);
		}
		lib_write("\r                                                             \r");
	}

	/**************************************************************************/
	/* Matches the string against a mask.                                     */
	/**************************************************************************/
	inline bool lib_match(const char *str, const char *mask) {
		char *s,*m;
		if (!strcmp(mask, str)) return true;
		s = (char *)str;
		m = (char *)mask;
		while (*s && *m) {
			// Continue on character match
			if (*s == *m) {s++; m++; continue;}

			// Continue on ? wildcard
			if (*m == '?') {s++; m++; continue;}

			// Check for * wildcard, recursively for each possible matching
			if (*m == '*') {
				m++;
				if (!*m) return true;
				while (*s && *m && *m != *s) s++;
				if (!*s) return false;
				if (lib_match(s,m)) return true;
				s++; m--;
				continue;
			}

			if (*m != *s) return false;
		}
		return (!*s && !*m);
	}

	/**************************************************************************/
	/* Checks if entry is a valid directory.                                  */
	/**************************************************************************/
	inline int lib_isdir(const char *entry) {
		struct stat s;
		if (!entry) return false;
		if (stat(entry, &s)) return false;
		if (!(s.st_mode & S_IFDIR)) return false;
		return true;
	}

	/**************************************************************************/
	/* Checks if path is a regular file,                                      */
	/**************************************************************************/
	inline int lib_isfile(const char *entry) {
		struct stat s;
		if (!entry) return false;
		if (stat(entry, &s)) return false;
		if (!(s.st_mode & S_IFREG)) return false;
		return true;
	}

	/**************************************************************************/
	/* Creates a list of filenames recursively browsing the subdirs too.      */
	/**************************************************************************/
	inline int lib_filelist(const char *dirname, const char *mask, char ***list) {
		int nitems, allocated;
		char dn[256];

		#if defined WIN32
			#if _MSC_VER >= 1400
				intptr_t d;
			#else
				int d;
			#endif
			struct _finddata_t de;
			#define DE_NAME de.name
		#elif defined LINUX
			DIR *d;
			struct dirent *de;
			#define DE_NAME de->d_name
		#endif
		#define FREE_LIST(items,nitem) {if(items){ int i; for(i=0;i<nitem;i++)if(items[i])free(items[i]); free(items); items=0; }}

		if (!list) return -1;
		*list = NULL;
		nitems = 0;
		allocated = 0;
		if (!lib_isdir(dirname)) return -1;

		#if defined WIN32
			gx_snprintf(dn, sizeof(dn), "%s\\*.*", dirname);
			d = _findfirst(dn, &de);
			do {
		#elif defined LINUX
			d = opendir(dirname);
			if (!d) return -1;
			while ((de = readdir(d))) {
		#endif
				if (!strcmp(DE_NAME, ".") || !strcmp(DE_NAME, "..")) continue;
				gx_snprintf(dn, sizeof(dn), "%s/%s", dirname, DE_NAME);
				dn[sizeof(dn)-1]=0;

				if (lib_isdir(dn)) {
					char **_list = NULL;
					int _nitems;
					_nitems = lib_filelist(dn, mask, &_list);
					if (_nitems < 0) {
						FREE_LIST(*list,nitems);
						return -1;
					} else if(!allocated){
						allocated=_nitems;
						nitems=_nitems;
						*list=_list;
					} else {
						if (allocated < (nitems + _nitems)) {
							char **__list = (char **)realloc(*list, (nitems + _nitems + 32) * sizeof(char*));
							if (!__list) {
								FREE_LIST(*list,nitems);
								FREE_LIST(_list,_nitems);
								return -1;
							}
							*list = __list;
							allocated = nitems+_nitems+32;
						}
						memcpy((*list)+nitems,_list,_nitems*sizeof(char*));
						nitems+=_nitems;
						if (_list) free(_list);
					}
				} else {
					if (lib_match(DE_NAME, mask)) {
						if (allocated <= nitems) {
							char **__list = (char **)realloc(*list, (nitems + 32) * sizeof(char*));
							if (!__list) {
								FREE_LIST(*list,nitems);
								return -1;
							}
							*list = __list;
							allocated = nitems + 32;
						}
						(*list)[nitems++] = strdup(dn);
					}
				}

		#if defined LINUX
			}
			closedir(d);
		#elif defined WIN32
			} while (_findnext(d, &de) != -1);
			_findclose(d);
		#endif
		#undef DE_NAME
		#undef FREE_LIST

		return nitems;
	}

/******************************************************************************/
#endif /* __LIB__INCL */
/******************************************************************************/
