/** \file *********************************************************************
 *
 * pr_barcode - Passport Reader sample program
 *
 * 2006-2021 (c) Adaptive Recognition (https://adaptiverecognition.com)
 ******************************************************************************/

/**
 * Image capturing and barcode reading
 *
 * Purpose:
 *		Show how to capture images and to read barcode.
 *
 * Description:
 *
 *		Capture images after successful startup.
 *		Read the barcode, the calculated checksum, and display the result.
 *
 */

#include "prapi.h"
#include "gxsdldr.c"

#include "../__lib__.h"

/**
 * Main Function
 */
int main() {
	char *ec[]={"Ok","Warning","Error"};
	gxVARIANT img;
	gxIMAGE *gim;
	gxVARIANT pdoc = NULL;

	/* Opening the PR system */
	gxHANDLE hpr;	/* handle for the PR system */
	lib_function_start("Opening system files");
	if (gx_openmodulea(&hpr,"prapi","default")) lib_function_end();
	else {
		lib_displ_err();
		return 0;
	}

	/* Connecting to the first device */
	lib_function_start("Connecting to the first device");
	if (pr_usedevicen(hpr,0,PR_UMODE_FULL_CONTROL)) {
		lib_function_end();
	} else {
		lib_displ_err();
		gx_closehandle(&hpr);
		return 0;
	}

	while(!lib_kbhit()) {

		int code;
		int status;
		char path[100], text[3000];
		int stext = sizeof(text);
		int i, id = PRV_FIELDLIST;
		gxVARIANT fieldlist;

		lib_process_start("Processing document");

		/* Capturing images */
		lib_function_start("Capturing images");
		if (!pr_capture(hpr)) {
			lib_displ_err();
		} else {
			lib_function_end();

			/* Reading barcode */
			lib_function_start("Reading barcode from infra image");
			if (pr_getbarcode(hpr,0,PR_LIGHT_INFRA,PR_IT_ORIGINAL,0,0,&pdoc))
				lib_function_end();
			else
				lib_displ_err();

			if (!pdoc) {
				lib_function_start("Reading barcode from white image");
				if (pr_getbarcode(hpr,0,PR_LIGHT_WHITE,PR_IT_ORIGINAL,0,0,&pdoc))
					lib_function_end();
				else
					lib_displ_err();
			}

			if (!pdoc) {
				lib_write_line("No data found!");
			} else {
				/* Displaying document type */
				if (pr_getdocumentstatus(hpr, pdoc, &code, &status)) {
					lib_write_line("Document type: %i, status: %s", code, ec[status/100]);
				} else {
					lib_displ_err();
				}

				/* Searching for the barcode and displaying it */
				if (pr_getfieldfromdoca(hpr, pdoc, PR_DF_BC1, text, &stext, &status, 0) ||
					pr_getfieldfromdocb(hpr, pdoc, PR_DF_BC1, text, &stext, &status, 0)) {
					int type = -1;
					int stype = 4;

					text[stext] = 0;

					/* Get the barcode type */
					if(!gx_convertvariantchild(pdoc, PRV_BARCODE, 0, GX_VARIANT_INT, &stype, 0, &type, stype))
						lib_displ_err();

					/* Converting the data for displaying on console */
#ifdef WIN32
					CharToOemBuff(text, text, stext);
#endif

					lib_write_line("Barcode type: %i, status: %i, length: %i\ndata:\n%s", type, status, stext, text);

				} else {
					lib_displ_err();
				}

				/* Saving the barcode image */
				lib_function_start("Save barcode image");
				if(pr_getfieldimage(hpr,pdoc,PR_DF_BC1,0,&img) &&
					gx_imagefromvariant(gx_direct(GX_CALL_GROUP_GX_IMAGE), &gim, img) &&
					gx_saveimagea(gx_direct(GX_CALL_GROUP_GX_IMAGE), gim, "barcode.jpg", GX_JPEG) &&
					gx_unrefimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), gim) &&
					gx_disposevariant(&img))
				{
					lib_function_end();
				} else {
					lib_displ_err();
				}

				/* Searching for the fields and displaying them */
				if(gx_getvariantchild(pdoc,GX_VARIANT_BY_ID, &id, 0, &fieldlist)) {
					int nitems = 0;
					/* Get the number of recognized fields */
					if(gx_getvariant(fieldlist, 0, 0, 0, &nitems, 0)) {
						for (i=0; i<nitems; i++) {
							int field_code = 0;
							int sz = sizeof(int);
							int n = 0;
							gx_snprintf(path, sizeof(path), "L,X=%d", i);
							if(gx_convertvariantbypatha(fieldlist, path, 0, GX_VARIANT_INT, &sz, &n, &field_code, sizeof(field_code))) {
								/* Get the field text */
								stext = sizeof(text);

								if (pr_getfieldfromdoca(hpr,pdoc,field_code,text,&stext,&status,0) ||
									pr_getfieldfromdocb(hpr,pdoc,field_code,text,&stext,&status,0)) {
									if (*text) {
										text[stext] = 0;
										/* Converting the data for displaying on console */
#ifdef WIN32
										CharToOem(text,text);
#endif
										lib_write_line("[%5i] \"%s\" [%s]", field_code, text, ec[status/100]);
									} else {
										lib_write_line("[%5i] \"Binary or empty data...\" [%s]", field_code, ec[status/100]);
									}
								} else {
									lib_displ_err();
								}

							} else {
								lib_displ_err();
							}
						}
					}
					if (!gx_disposevariant(&fieldlist)) lib_displ_err();
				}
				if (!gx_disposevariant(&pdoc)) lib_displ_err();
			}
		}

		lib_process_end();
		lib_wait_for_sec(3);
	}

	/* Closing the device */
	lib_function_start("Closing the device");
	if (pr_closedevice(hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	/* Closing the PR system */
	lib_function_start("Closing the PR system");
	if (gx_closehandle(&hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	return lib_print_stat();
}
