/** \file *********************************************************************
 *
 * pr_capture_adv - Passport Reader sample program
 *
 * 2006-2021 (c) Adaptive Recognition (https://adaptiverecognition.com)
 ******************************************************************************/

/**
 * Advanced image capturing
 *
 * Purpose:
 *		Show how to capture image using advanced features.
 *
 * Description:
 *		Capture images after successful startup. Set the event function, and handle the events
 *		during the capture process. At the end save the original and the document image.
 *
 */

#include "prapi.h"
#include "gxsdldr.c"

#include "../__lib__.h"

/**
 * Displays event messages
 */
void GXAPI evfunc(int event, int param1, int param2, void *uparam)
{
	switch (event) {
		case PR_EV_UVLEDWARMING:
			lib_write_line("<Event> UV tube warming started");
			break;
		case PR_EV_PAGECAPTURED:
			lib_write_line("<Event> Page captured: page = %i, error_code = %i", param1, param2);
			break;
		case PR_EV_IMAGECAPTURED:
			lib_write_line("<Event> Image captured: page = %i, light = %i", param1, param2);
			break;
		case PR_EV_CAPTURESTARTED:
			lib_write_line("<Event> Capture started: page = %i", param1);
			break;
		case PR_EV_NOIMAGEINFO:
			lib_write_line("<Event> No image info: page = %i, light = %i", param1, param2);
			break;
		case PR_EV_DOCFRAMEFOUND:
			lib_write_line("<Event> Document frame found: page = %i", param1);
			break;
		default:
			lib_write_line("<Event> Other event(%i): p1 = %i, p2 = %i", event, param1, param2);
			break;
	}
}

/**
 * Main Function
 */
int main() {
	char **devlist; /* list of the devices */
	int ndev;	/* number of devices */
	unsigned int pages[1]={0xff};

	/* Opening the PR system */
	gxHANDLE hpr;	/* handle for the PR system */
	lib_function_start("Opening system files");
	if (gx_openmodulea(&hpr,"prapi","default")) lib_function_end();
	else {
		lib_displ_err();
		return 0;
	}

	/* Getting the device list */
	lib_function_start("Getting the device list");
	if (pr_getdevicelista(hpr,&devlist,&ndev)) {
		lib_function_end();
	} else {
		lib_displ_err();
		gx_closehandle(&hpr);
		return 0;
	}

	if (ndev <= 0) {
		lib_write_line("No device found!");
		return 0;
	} else {
		lib_write_line("%i device(s) found!", ndev);
	}

	/* Connecting to the first device */
	lib_function_start("Connecting to the first device %s",devlist[0]);
	if (pr_usedevicea(hpr,devlist[0],PR_UMODE_FULL_CONTROL)) {
		lib_function_end();
	} else {
		lib_displ_err();
		gx_closehandle(&hpr);
		return 0;
	}

	/* Warming UV tubes */
	lib_function_start("Warming UV tubes");
	if (gx_setmodulepropertyinta(hpr,"freerun_mode",PR_FRMODE_UVLEDWARMING)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	/* Setting all the lights for the first page */
	lib_function_start("Setting lights");
	if (pr_setpagelight(hpr,1,pages)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	/* Setting event function */
	lib_function_start("Setting event function");
	if (gx_setmodulepropertyinta(hpr,"event_types",PR_ET_LED|PR_ET_CAPTURE) &&
		pr_seteventfunction(hpr,evfunc,(void*)0)) {
			lib_function_end();
	} else {
		lib_displ_err();
	}

	while(!lib_kbhit()) {

		lib_process_start("Processing document");

		/* Capturing images */
		lib_function_start("Capturing images");
		if (pr_capture(hpr)) {
			lib_function_end();
		} else {
			lib_displ_err();
			break;
		}

		/* Saving to file */
		lib_function_start("Saving original image");
		if (pr_saveimagea(hpr,0,PR_LIGHT_WHITE,PR_IT_ORIGINAL,"original.jpg",GX_JPEG)) {
			lib_function_end();
		} else {
			lib_displ_err();
		}

		lib_function_start("Saving document image");
		if (pr_saveimagea(hpr,0,PR_LIGHT_WHITE,PR_IT_DOCUMENT,"document.jpg",GX_JPEG)) {
			lib_function_end();
		} else {
			lib_displ_err();
		}

		lib_process_end();
		lib_wait_for_sec(3);
	}

	/* Turning off the lights */
	lib_function_start("Turning off the lights");
	if (gx_setmodulepropertyinta(hpr,"freerun_mode",PR_FRMODE_NULL)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	/* Closing the device */
	lib_function_start("Closing the device");
	if (pr_closedevice(hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	/* Destroying the devlist array */
	if (!gx_globalfree(devlist)) lib_displ_err();

	/* Closing the PR system */
	lib_function_start("Closing the PR system");
	if (gx_closehandle(&hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	return lib_print_stat();
}
