/** \file *********************************************************************
 *
 * pr_motdet - Passport Reader sample program
 *
 * 2006-2021 (c) Adaptive Recognition (https://adaptiverecognition.com)
 ******************************************************************************/

/**
 * Device controlling example
 *
 * Purpose:
 *		Show how to control the device.
 *
 * Description:
 *		Test the power of the device after successful startup.
 *
 *		Enable motion detection and check the state of the start button and the motion detection, and display it
 *		on the console. The display leds are controlled by the result of the tests.
 *
 *		Press a key to break the testing loop.
 */

#include "prapi.h"
#include "gxsdldr.c"

#include "../__lib__.h"

/**
 * Main function
 */
int main() {
	int state;
	char *statxt[]={"No document detected.","Document is moving.",
		"Take out the document!","","Ready for reading.","Button is pressed."};

	/* Opening the PR system */
	gxHANDLE hpr;	/* handle for the PR system */
	lib_function_start("Opening system files");
	if (gx_openmodulea(&hpr,"prapi","default")) lib_function_end();
	else {
		lib_displ_err();
		return 0;
	}

	/* Connecting to the first device */
	lib_function_start("Connecting to the first device");
	if (pr_usedevicen(hpr,0,PR_UMODE_FULL_CONTROL)) {
		lib_function_end();
	} else {
		lib_displ_err();
		gx_closehandle(&hpr);
		return 0;
	}

	/* Testing power switch */
	lib_function_start("Testing the power switch");
	if (pr_testpower(hpr,&state,0)) {
		if ((state & PR_PWR_MASK) == PR_PWR_OFF)
			lib_write_line("Power is off! Please turn power on!");

		while ((state & PR_PWR_MASK) == PR_PWR_OFF) {
			if (!pr_testpower(hpr,&state,0)) lib_displ_err();
			lib_wait(300);
		}

		lib_function_end();
	} else {
		lib_displ_err();
	}

	/* Enabling motion detection */
	lib_function_start("Enabling motion detection");
	if (gx_setmodulepropertyinta(hpr,"freerun_mode",PR_FRMODE_TESTDOCUMENT)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	while(!lib_kbhit()) {
		/* Testing the start button */
		if (pr_testbutton(hpr,PR_KEY_START,&state)) {
			if (state == PR_KEY_PRESSED) state=5;
			else {
				/* If the start button is not pressed testing the document detection */
				if (pr_testdocument(hpr,&state,0)) {

					/* Turning the display leds depending on the status */
					int color = PR_SLC_OFF;
					switch(state) {
						case PR_TD_OUT:		color = PR_SLC_GREEN; break;
						case PR_TD_MOVE:	color = PR_SLC_ANY; break;
						case PR_TD_NOMOVE:	color = PR_SLC_RED; break;
					}
					if (!pr_setstatusled(hpr, 0xFF, color)) lib_displ_err();
				} else {
					lib_displ_err();
				}
			}
			lib_write("\rState of the device: %s        \r", statxt[state]);
			lib_wait(200);
			if ((state == PR_TD_IN) || (state == 5)) lib_wait(800);
		} else {
			lib_displ_err();
		}
	}
	lib_write_line("");

	/* Turning off the lights */
	lib_function_start("Turning off the lights");
	if (gx_setmodulepropertyinta(hpr, "freerun_mode", PR_FRMODE_NULL)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	/* Turning off the display leds */
	lib_function_start("Turning off the display leds");
	if (pr_setstatusled(hpr, 0xFF, PR_SLC_OFF)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	/* Closing the device */
	lib_function_start("Closing the device");
	if (pr_closedevice(hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	/* Closing the PR system */
	lib_function_start("Closing the PR system");
	if (gx_closehandle(&hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	return 1;
}
