/** \file *********************************************************************
 *
 * pr_rfid - Passport Reader sample program
 *
 * 2006-2021 (c) Adaptive Recognition (https://adaptiverecognition.com)
 ******************************************************************************/

/**
 * RFID handling example
 *
 * Purpose:
 *		Show how to read data from an RFID chip.
 *
 * Description:
 *
 *		Activate the Basic Access Control mechanism in case of BAC protected documents.
 *		Reads DG1 and DG2 files from the RFID chip.
 *
 */

#include "prapi.h"
#include "gxsdldr.c"

#include "../__lib__.h"

/* The make_bac helper function, reading MRZ from the document and using it to make BAC */
int make_bac(gxHANDLE hpr, char* card)
{
	int j,l;
	gxVARIANT pdoc;
	int pages;
	char *mrz[3];

	/* Setting only infra light for capture */
	lib_function_start("\t- Setting infra light");
	if (pr_getlightmask(hpr,&pages,PR_LIGHT_INFRA,0) && pr_setpagelight(hpr,1,(gxu32*)&pages)) lib_function_end();
	else return lib_displ_err();

	/* Capturing images */
	lib_function_start("\t- Capturing images");
	if (pr_capture(hpr)) lib_function_end();
	else return lib_displ_err();

	/* Reading MRZ */
	lib_function_start("\t- Reading MRZ");
	if (pr_getmrz(hpr, 0, PR_LIGHT_INFRA, PR_IT_ORIGINAL, &pdoc)) lib_function_end();
	else return lib_displ_err();

	if (!pdoc) {printf("No MRZ data found!\n"); return -1;}

	for (j=0; j<3; j++) {
		mrz[j] = (char*) malloc(45);
		memset(mrz[j], 0 ,45);
		l = 45;
		pr_getfieldfromdoca(hpr, pdoc, PR_DF_MRZ1 + j, mrz[j], &l, 0, 0);
	}

	gx_disposevariant(&pdoc);

	/* Making BAC */
	lib_function_start("\t- Making BAC");
	if (pr_makebaca(hpr, card, (const char **)mrz, strlen(mrz[2]) ? 3 : 2)) {
		for (j=0; j<3; j++) free(mrz[j]);
		lib_function_end();
	} else {
		for (j=0; j<3; j++) free(mrz[j]);
		return lib_displ_err();
	}

	return 0;
}

/* The resolve_dg1 helper function, parsing EF_DG1 and displaying MRZ information */
int resolve_dg1(gxHANDLE hpr, void * data, int sdata)
{
	int error_code = 0;
	gxVARIANT pdoc;
	char text[256];
	int stext;

	lib_function_start("\t- Resolving DG1");
	if (pr_resolverfiddata(hpr, data, sdata, &pdoc)) {
		if (!pdoc) lib_error("No MRZ data found!");
		else {
			lib_function_end();
			lib_write_line("\t- MRZ data:");
			stext = sizeof(text);
			if (pr_getfieldfromdoca(hpr, pdoc, PR_DF_RFID_MRZ, text, &stext, NULL, 0)) {
				char s[45];
				gx_snprintf(s, 44, text);
				s[44] = 0;
				lib_write_line("\t\t %s", s);
				gx_snprintf(s, 44, text+44);
				s[44] = 0;
				lib_write_line("\t\t %s", s);
			} else {
				error_code = lib_displ_err();
			}
			if (pdoc) gx_disposevariant(&pdoc);
		}
	} else {
		error_code = lib_displ_err();
	}

	return error_code;
}

/* The resolve_dg2 helper function, parsing EF_DG2 and saving the face image */
int resolve_dg2(gxHANDLE hpr, void * data, int sdata)
{
	int error_code = 0;
	gxVARIANT pdoc;
	gxIMAGE *img = NULL;

	lib_function_start("\t- Resolving");
	if (pr_resolverfiddata(hpr, data, sdata, &pdoc)) {
		if (!pdoc) lib_error("No image data found!");
		else {
			lib_function_end();
			lib_function_start("\t- Saving image");
			if (pr_getgximagefromdoc(hpr, pdoc, PR_DF_RFID_FACE, &img)) {
				if (gx_saveimagea(gx_direct(GX_CALL_GROUP_GX_IMAGE), img, "face.jpg", GX_JPEG)) lib_function_end();
				else error_code = lib_displ_err();
				gx_unrefimage(gx_direct(GX_CALL_GROUP_GX_IMAGE), img);
			} else {
				error_code = lib_displ_err();
			}
			if (pdoc) gx_disposevariant(&pdoc);
		}
	} else {
		error_code = lib_displ_err();
	}

	return error_code;
}

/**
 * Main function
 */
int main() {
	int i;
	char ** cardlist=0;
	int numcards=0;
	void * data=NULL;
	int sdata = 0, srfid_files = 4;
	unsigned int rfid_files = 0;
	char path[64], fn[32];
	gxVARIANT pdoc = NULL;

	/* Opening the PR system */
	gxHANDLE hpr;		/* handle for the PR system */
	lib_function_start("Opening system files");
	if (gx_openmodulea(&hpr,"prapi","default")) lib_function_end();
	else {
		lib_displ_err();
		return 0;
	}

	/* Connecting to the first device */
	lib_function_start("Connecting to the first device");
	if (pr_usedevicen(hpr,0,PR_UMODE_FULL_CONTROL)) {
		lib_function_end();
	} else {
		lib_displ_err();
		gx_closehandle(&hpr);
		return 0;
	}

	while(!lib_kbhit()) {

		lib_process_start("RFID");

		/* Searching for RFID cards */
		lib_function_start("Searching for RFID cards ...");
		if (pr_getrfidcardlista(hpr, &cardlist, &numcards)) {
			lib_function_end();

			if (numcards == 0) {
				lib_write_line("No card found");
			} else {
				lib_write_line("");

				do {
					/* Connecting to the first chip */
					lib_function_start("Connecting to the RFID chip");
					if (pr_connectrfidcarda(hpr, cardlist[0])) lib_function_end();
					else {
						lib_displ_err();
						continue;
					}

					lib_write_line("Serial no.: %s", cardlist[0]);

					/* Initializing chip reading */
					lib_function_start("Initializing chip reading");
					if (pr_getrfidfilea(hpr, cardlist[0], PR_RFID_EF_COM, &data, &sdata)) lib_function_end();
					else {
						lib_displ_err();
						if (gx_geterrorcode() != GX_EACCES) continue;
						if(make_bac(hpr, cardlist[0])) continue;
						lib_function_start("Initializing chip reading");
						if (pr_getrfidfilea(hpr, cardlist[0], PR_RFID_EF_COM, &data, &sdata)) lib_function_end();
						else {lib_displ_err(); continue;}
					}

					/* Getting available files */
					lib_function_start("\t- Available files");
					gx_snprintf(path, sizeof(path), "C,D=%i/L,V=%i/C,D=%i", PRV_FIELDLIST, PR_DF_RFID_FIELDS|PR_DFE_DOCUMENT_DESCRIPTOR, PRV_RFIDDIR);
					if (
						!pr_resolverfiddata(hpr, data, sdata, &pdoc) ||
						!gx_convertvariantbypatha(pdoc, path, 0, GX_VARIANT_INT, &srfid_files, 0, &rfid_files, srfid_files)
						) {
							lib_displ_err();
							if (pdoc) gx_disposevariant(&pdoc);
							if (data) {gx_globalfree(data); data = NULL; sdata = 0;}
							continue;
					}
					if (pdoc) gx_disposevariant(&pdoc);
					if (data) {gx_globalfree(data); data = NULL; sdata = 0;}

					for (i=PR_RFID_EF_DG1; i<=PR_RFID_EF_DG16; i++)
						if (((1<<i) & rfid_files)) lib_write("DG%i ", i);
					lib_function_end();

					/* Reading available files, saving and resolving them */
					for (i=PR_RFID_EF_DG1; i<=PR_RFID_EF_DG2; i++) {
						if (!((1<<i) & rfid_files)) continue;
						gx_snprintf(fn, 32, "EF_DG%i.dat", i);
						lib_function_start("Reading %s", fn);
						if (pr_getrfidfilea(hpr, cardlist[0], i, &data, &sdata)) lib_function_end();
						else {lib_displ_err(); continue;}
						lib_function_start("\t- Saving %s", fn);
						if (!lib_save(fn, data, sdata)) lib_error("File write error!");
						else lib_function_end();
						if (i == PR_RFID_EF_DG1) resolve_dg1(hpr, data, sdata);
						if (i == PR_RFID_EF_DG2) resolve_dg2(hpr, data, sdata);
						if (data) {gx_globalfree(data); data = NULL; sdata = 0;}
					}

					/* Disconnecting from the chip */
					lib_function_start("Disconnecting from the RFID chip");
					if (pr_disconnectrfidcarda(hpr, cardlist[0])) lib_function_end();
					else lib_displ_err();

				} while(false);

			}
			if (cardlist) {gx_globalfree(cardlist); cardlist = NULL;}
		} else {
			lib_displ_err();
		}

		lib_process_end();
		lib_wait_for_sec(3);
	}

	/* Closing the device */
	lib_function_start("Closing the device");
	if (pr_closedevice(hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	/* Closing the PR system */
	lib_function_start("Closing the PR system");
	if (gx_closehandle(&hpr)) {
		lib_function_end();
	} else {
		lib_displ_err();
	}

	return lib_print_stat();
}
