#ifndef PR22_FPS_INCL
#define PR22_FPS_INCL
/******************************************************************************/
/**	\file
 *	Central class for a fingerprint scanner device.
 */
/******************************************************************************/
#define PR22_VERSION 2.2.0.20
#include "Imaging/FingerScanner.hpp"
#include "Control/Peripherals.hpp"
#include "Events/Events.hpp"
#include "PrIns/gxMutex.hpp"
//------------------------------------------------------------------------------
namespace Pr22 {

/** Central class for a fingerprint scanner device.
 *
 * A device is composed of several modules. These modules can be
 * accessed through this class.
 */
class FingerprintScannerDevice {
public:

	inline FingerprintScannerDevice();										// Initializes the Fingerprint Scanner software.
	inline FingerprintScannerDevice(const Util::InputString & proppath);	// Initializes the Fingerprint Scanner software.
	inline static std::vector<PrOutputString> GetDeviceList(void);			// Collects a list of device names.
	inline FingerprintScannerDevice & UseDevice(int devno);					// Starts using a device.
	inline FingerprintScannerDevice & UseDevice(const Util::InputString & devname);	// Starts using a device.
	inline PrOutputString GetDeviceName();									// Returns the name of the device.
	inline PrOutputString GetVersion(char component);						// Returns version information of the API.
	inline void Close(void);												// Ends using of the current device.
	inline void SetProperty(const Util::InputString & property, const Util::InputString & value);	// Sets a property value of the Fingerprint Scanner system.
	inline PrOutputString GetProperty(const Util::InputString & property);	// Returns a property value of the Fingerprint Scanner system.
	inline Peripherals GetPeripherals();									// Returns the device peripheral controller.
	inline FingerScanner & GetScanner();									// Returns the finger scanner.
	inline void AddEventHandler(const Events::Event & event, Events::FingerEventHandler * handler);	// Sets an event handler object.
	inline void RemoveEventHandler(const Events::Event & event, Events::FingerEventHandler * handler);	// Removes an event handler object.
	inline void TriggerEvent(const Events::Event & event);					// Triggers an event.

private:

	PrIns::gxModule ApiHandle;			// Api handler.
	std::vector<FingerScanner> scanner;	// The scanner module.
	int event_filter;					// Used events.
	std::vector<Events::FingerEventHandler*> events[19];	// The registered event handlers.
	PrIns::Mutex evlsmx, evarmx;		// Event mutex.

	inline void init(const wchar_t * proppath);				// Initializes the class.
	inline static void GXAPI EventFunction(int event, int param1, int param2, void* uparam);	// Global event function.
};

/******************************************************************************
 * This is the end of the FingerprintScannerDevice interface. Now the implementation follows
 ******************************************************************************/

/// Initializes the Fingerprint Scanner software.
FingerprintScannerDevice::FingerprintScannerDevice() : event_filter(0) {
	init(L"default");
}

/** Initializes the Fingerprint Scanner software with a specified property path.
 *
 * Specified path means the initial data are read from the gxsd.dat file
 * from the specified xml node.
 *
 * @param proppath	Name of the xml node of the gxsd.dat file from
 *					where the initial data should be read.
 */
FingerprintScannerDevice::FingerprintScannerDevice(const Util::InputString & proppath) : event_filter(0) {

	init(proppath.GetString());
}

// Initializes the class.
void FingerprintScannerDevice::init(const wchar_t * proppath) {

	ApiHandle = PrIns::gxModule(L"prapi", proppath);
	#ifndef PR22_APILANG
		ApiHandle.SetProperty(L"sdk", GXM_I2S(PR22_VERSION) " [C++]");
	#else
		ApiHandle.SetProperty(L"sdk", GXM_I2S(PR22_VERSION) " [" GXM_I2S(PR22_APILANG) "]");
	#endif
	ApiHandle.SetProperty(L"event_filter", Util::ToString(event_filter));
	ApiHandle.SetProperty(L"async_callback", L"1");
	PrIns::prSetEventFunction sef = { (void*)&EventFunction, this };
	ApiHandle.Call(PrIns::ApiCallCodes::SetEventFunction, &sef);
}

/** Collects a list of device names for UseDevice(const Util::InputString & devname) method.
 *
 * @return			List of device names.
 */
std::vector<PrOutputString> FingerprintScannerDevice::GetDeviceList(void) {
	std::vector<PrOutputString> devices;
	PrIns::CommonGetDeviceList gdl = { L"FPS", 0, 0 };
	PrIns::gxModule prHandle(L"prapi", L"default");
	if(prHandle.Call(PrIns::ApiCallCodes::GetDeviceList, &gdl))
		return devices;
	for(int i=0; i<gdl.ndevices; i++) {
		devices.push_back(ToPrOutputString(((wchar_t**)gdl.devices)[i]));
	}
	PrIns::GlobalData(gdl.devices);
	return devices;
}

/** Starts using a device.
 *
 * @param devno		Ordinal number of the device to use.
 * @return			This object.
 */
FingerprintScannerDevice & FingerprintScannerDevice::UseDevice(int devno) {
	std::vector<PrOutputString> DevNames = GetDeviceList();
	if(devno<0) {
		throw Exceptions::InvalidParameter(L"[pr22] (devno)");
	}
	if(devno>=(int)DevNames.size()) {
		throw Exceptions::NoSuchDevice(L"[pr22] (devno)");
	}
	return UseDevice(DevNames[(size_t)devno]);
}

/** Starts using a device.
 *
 * The list of available names can be requested from the GetDeviceList().
 *
 * @param devname	Name of the device to use.
 * @return			This object.
 */
FingerprintScannerDevice & FingerprintScannerDevice::UseDevice(const Util::InputString & devname) {
	scanner.clear();

	PrIns::CommonUseDevice param = { devname.GetString(), 2 /* FULL CONTROL */ };
	ApiHandle.Call(PrIns::ApiCallCodes::UseDevice, &param);

	scanner.push_back(PrIns::Invoker().Construct(ApiHandle, (FingerScanner*)0));

	return *this;
}

/// Returns the name of the device.
PrOutputString FingerprintScannerDevice::GetDeviceName() {
	try {
		PrIns::CommonGetDeviceInfo gdi = { 0 };
		if(ApiHandle.Call(PrIns::ApiCallCodes::GetDeviceInfo, &gdi))
			return PrOutputString();
		return Util::Variant(gdi.DevInfo)[0].ToString();
	}
	catch(const Exceptions::General &) {
		return PrOutputString();
	}
}

/** Returns version information of the API.
 *
 * @param component	\li Character @c 'A' for version of the interface files,
 *					\li or character @c 'S' for version of the Fingerprint Scanner system.
 * @return			Version string.
 */
PrOutputString FingerprintScannerDevice::GetVersion(char component) {
	if(component=='A')
		return ToPrOutputString(GXM_CNC(L, GXM_I2S(PR22_VERSION)));
	if(component=='S')
		return ApiHandle.GetProperty(L"api_version");
	return ToPrOutputString(L"-");
}

/// Ends using of the current device.
void FingerprintScannerDevice::Close(void) {
	scanner.clear();
	if(ApiHandle.IsValid()) {
		PrIns::CommonUseDevice ud = { 0, 0 };
		ApiHandle.Call(PrIns::ApiCallCodes::UseDevice, &ud);
	}
}

/** Sets a property value of the Fingerprint Scanner system.
 *
 * The names and values of the properties are documented in the
 * Fingerprint Scanner reference manual.
 *
 * @param property	The name of the property.
 * @param value		The value of the property.
 */
void FingerprintScannerDevice::SetProperty(const Util::InputString & property, const Util::InputString & value) {
	ApiHandle.SetProperty(property, value);
}

/** Returns a property value of the Fingerprint Scanner system.
 *
 * The names and values of the properties are documented in the
 * Fingerprint Scanner reference manual.
 *
 * @param property	The name of the property.
 * @return			The value of the property.
 */
PrOutputString FingerprintScannerDevice::GetProperty(const Util::InputString & property) {
	return ApiHandle.GetProperty(property);
}

/** Returns the device peripheral controller module of the used
 * FingerprintScannerDevice.
 */
Peripherals FingerprintScannerDevice::GetPeripherals() {
	return PrIns::Invoker().Construct(ApiHandle, (Peripherals*)0);
}

/// Returns the finger scanner module of the used FingerprintScannerDevice.
FingerScanner & FingerprintScannerDevice::GetScanner() {
	if(!scanner.size()) throw Exceptions::NoSuchDevice(L"[pr22] (Device not opened)");
	return scanner[0];
}

void GXAPI FingerprintScannerDevice::EventFunction(int event, int param1, int param2, void* uparam) {

	FingerprintScannerDevice *fsd = (FingerprintScannerDevice*)uparam;
	if(event<0 || event>=(int)gx_arraycnt(fsd->events)) return;

	fsd->evlsmx.Lock();
	std::vector<Events::FingerEventHandler*> hlist = fsd->events[event];
	fsd->evlsmx.Unlock();

	using Events::Event;
	for(size_t ix=0; ix<hlist.size(); ix++) {

		if(event == Event::ImageScanned)
			hlist[ix]->OnFingerImageUpdated((Imaging::FingerPosition)param2);
		if(event == Event::ScanFinished)
			hlist[ix]->OnScanFinished();
		if(event == Event::PresenceDetection)
			hlist[ix]->OnPresenceStateChanged((Util::PresenceState)param1);
		if(event == Event::PreviewCaptured)
			hlist[ix]->OnPreviewCaptured();
		if(event == Event::Button)
			hlist[ix]->OnKeyPressed(param1, param2);
		if(event == Event::Power)
			hlist[ix]->OnPowerChanged(param1, param2);
		if(event == Event::Connection)
			hlist[ix]->OnConnection(param1);
		if(event == Event::FingerPositioning)
			hlist[ix]->OnFingerPositioning(PrIns::Invoker().Construct(param2, (Processing::FingerPositioningFailure*)0));
		if(event == Event::DeviceUpdate)
			hlist[ix]->OnDeviceUpdate(param1);

	}
}

/** Sets an event handler object to a specified event.
 *
 * @param event		The specified Events::Event to handle.
 * @param handler	An object derived from Events::FingerEventHandler class that contains
 *					the involved event handler method.
 */
void FingerprintScannerDevice::AddEventHandler(const Events::Event & event, Events::FingerEventHandler * handler) {
	if(event<0 || event>=(int)gx_arraycnt(events) || !handler) return;

	PrIns::MxLocker lock(evarmx);
	bool no_old_handlers = events[event].empty();
	evlsmx.Lock();
	events[event].push_back(handler);
	evlsmx.Unlock();
	TriggerEvent(event);

	if(no_old_handlers) {
		event_filter |= 1 << event;
		if(ApiHandle.IsValid()) ApiHandle.SetProperty(L"event_filter", Util::ToString(event_filter));
	}
}

/** Removes an event handler object from a specified event.
 *
 * \note Because of the multithreading environment, pending events may be still raised
 * for a short period, even after their event handler was removed.
 *
 * @param event		The specified Events::Event to handle.
 * @param handler	A previously registered object.
 */
void FingerprintScannerDevice::RemoveEventHandler(const Events::Event & event, Events::FingerEventHandler * handler) {
	if(event<0 || event>=(int)gx_arraycnt(events)) return;

	PrIns::MxLocker lock(evarmx);
	std::vector<Events::FingerEventHandler*> &ev = events[event];
	std::vector<Events::FingerEventHandler*>::iterator it;
	evlsmx.Lock();
	for(it=ev.begin(); it!=ev.end(); ++it) if(*it==handler) { ev.erase(it); break; }
	evlsmx.Unlock();

	if(ev.empty()) {
		event_filter &= ~(1 << event);
		if(ApiHandle.IsValid()) ApiHandle.SetProperty(L"event_filter", Util::ToString(event_filter));
	}
}

/** Triggers an event.
 *
 * @param event		The specified Events::Event to trigger. Not all the events can be triggered.
 */
void FingerprintScannerDevice::TriggerEvent(const Events::Event & event) {
	if(event<0 || event>=(int)gx_arraycnt(events) || events[event].empty()) return;
	if(ApiHandle.IsValid()) ApiHandle.SetProperty(L"trigger_event", Util::ToString(1 << event));
}

} // namespace Pr22
//------------------------------------------------------------------------------
#endif //PR22_FPS_INCL
