#ifndef PR22_FINGERIMAGE_INCL
#define PR22_FINGERIMAGE_INCL
/******************************************************************************/
/**	\file
 *	Scanned finger image data.
 */
/******************************************************************************/
#define PR22_VERSION 2.2.0.20
#include "Imaging/RawImage.hpp"
#include "Imaging/Imaging.hpp"
#include "Util/Util.hpp"
//------------------------------------------------------------------------------
namespace Pr22 { namespace Imaging {

/// Scanned finger image data.
class FingerImage {
public:

	inline Util::Variant& GetVariant();					// Returns low level data of the image.
	inline ImpressionType GetImpressionType() const;	// Returns ImpressionType of the finger.
	inline FingerPosition GetFingerPosition() const;	// Returns FingerPosition of the finger.

	inline RawImage GetImage(void);					// Returns the RawImage object.
	inline int GetQuality(void) const;				// Returns the quality of the finger image.
	inline int GetDryWetQuality(void) const;		// Returns the humidity level of the finger image.
	inline int GetFraction(void) const;				// Returns the fraction level of the finger image.
	inline int GetNFIQ(void) const;					// Returns NIST fingerprint image quality.

private:

	PrIns::gxModule ApiHandle;			// Api handler.
	Util::Variant Image;				// Low level data of the image.
	ImpressionType Imptype;				// ImpressionType of the finger.
	FingerPosition Position;			// FingerPosition of the finger.

	friend class Pr22::PrIns::Invoker;
	FingerImage(const PrIns::gxModule & handle, const Util::Variant & image,
		const FingerPosition & position, const ImpressionType & mode)
		: ApiHandle(handle), Image(image), Imptype(mode), Position(position)
	{ }	// Constructor for internal usage only.
};

/******************************************************************************
 * This is the end of the FingerImage interface. Now the implementation follows
 ******************************************************************************/

/// Returns low level data of the image.
Util::Variant& FingerImage::GetVariant() {
	return Image;
}

/// Returns ImpressionType of the finger.
ImpressionType FingerImage::GetImpressionType() const {
	return Imptype;
}

/// Returns FingerPosition of the finger.
FingerPosition FingerImage::GetFingerPosition() const {
	return Position;
}

/// Returns the RawImage object.
RawImage FingerImage::GetImage(void) {
	return RawImage().FromVariant(Image);
}

/// Returns the quality of the finger image.
int FingerImage::GetQuality(void) const {
	return Image.GetChild(Util::VariantId::Confidence, 0).ToInt();
}

/// Returns the humidity level of the finger image.
int FingerImage::GetDryWetQuality(void) const {
	return Image.GetChild(Util::VariantId::FingerDryWet, 0).ToInt();
}

/// Returns the fraction level of the finger image.
int FingerImage::GetFraction(void) const {
	return Image.GetChild(Util::VariantId::FingerFraction, 0).ToInt();
}

/// Returns NIST fingerprint image quality.
int FingerImage::GetNFIQ(void) const
{
	if(Imptype == ImpressionType::Rolled) {
		throw Exceptions::InvalidFunction(L"[pr22] (rolled quality)");
	}
	try {
		return Image.GetChild(Util::VariantId::FingerNistQuality, 0).ToInt();
	}
	catch(const Exceptions::EntryNotFound &) { }

	struct fpsNFIQ {
		int Position;
		int Quality;
	} nfiq = { Position, 0 };
	ApiHandle.Call(PrIns::FpsApiCallCodes::NistFingerQuality, &nfiq);
	return nfiq.Quality;
}

} // namespace Imaging
} // namespace Pr22
//------------------------------------------------------------------------------
#endif //PR22_FINGERIMAGE_INCL
