#ifndef PR22_FINGERSCANNER_INCL
#define PR22_FINGERSCANNER_INCL
/******************************************************************************/
/**	\file
 *	Finger scanner module.
 */
/******************************************************************************/
#define PR22_VERSION 2.2.0.20
#include "Processing/FingerCollection.hpp"
#include "Imaging/FingerImage.hpp"
#include "Task/Task.hpp"
//------------------------------------------------------------------------------
namespace Pr22 {

/// Finger scanner module.
class FingerScanner {
public:

	FingerScanner() { }										// Default constructor
	/// Class for storing information on a FingerScanner.
	class Information {
	public:
		inline PrOutputString GetVersion(char component) const;	// Returns version information of the scanner.
		inline bool IsCalibrated() const;							// Returns if the scanner is calibrated.
		template<class TRECT> inline TRECT GetSize(int objectWindow, bool whtype = true) const;	// Returns the size of an object window of the scanner.

	private:

		PrIns::gxModule ApiHandle;							// Api handler.
		Util::Variant Info;									// Scanner information.

		friend class Pr22::PrIns::Invoker;
		inline Information(const PrIns::gxModule &handle);	// Constructor for internal usage only.
	};

	inline Information GetInfo() const;						// Returns the scanner information.
	inline Imaging::FingerImage GetFinger(const Imaging::FingerPosition & position, const Imaging::ImpressionType & mode);
	inline Processing::FingerCollection GetFingerCollection(const Task::FingerTask & task);	// Returns a set of previously scanned fingers to save.
	inline Imaging::RawImage GetLiveImage(void);			// Returns the last captured image.
	inline Task::TaskControl StartTask(const Task::FingerTask & task);	// Starts a task declared by the task parameter.
	inline void CleanUpData(void);							// Removes all the internally stored finger data from the system.

private:

	PrIns::gxModule ApiHandle;								// Api handler.

	friend class Pr22::PrIns::Invoker;
	FingerScanner(const PrIns::gxModule & handle) : ApiHandle(handle) { }	// Constructor for internal usage only.
};
/******************************************************************************
 * This is the end of the FingerScanner interface. Now the implementation follows
 ******************************************************************************/

// Constructor for internal usage only.
FingerScanner::Information::Information(const PrIns::gxModule & handle) : ApiHandle(handle) {
	PrIns::CommonGetDeviceInfo op = { 0 };
	ApiHandle.Call(PrIns::ApiCallCodes::GetDeviceInfo, &op);
	Info = Util::Variant(op.DevInfo).GetItemByPath(Util::ToString(L"L/V=%i", Util::HWType::Scanner));
}

/** Returns version information of the scanner.
 *
 * @param component	\li Character @c 'H' for version of the hardware, \li character @c 'F' for version of
 *					the firmware, \li or character @c 'S' for version of the device handling software library.
 * @return	Version information of the scanner.
 */
PrOutputString FingerScanner::Information::GetVersion(char component) const {
	try {
		return Info.GetChild(Util::VariantId::Version, 0).GetListItem(Util::VariantPos::ById(component)).ToString();
	}
	catch(const Exceptions::General &) {
		return ToPrOutputString(L"-");
	}
}

/** Returns if the scanner is calibrated.
 *
 * @return	True if the scanner is calibrated.
 */
bool FingerScanner::Information::IsCalibrated() const {
	int objectWindow = -1;
	PrIns::prIsCalibrated op = { (objectWindow + 1) << 24 };
	try {
		return !ApiHandle.Call(PrIns::ApiCallCodes::Iscalibrated, &op);
	}
	catch(const Exceptions::EntryNotFound &) {
		return false;
	}
}

/** Returns the size of an object window of the scanner.
 *
 * @param objectWindow	RFU. Set this value to 0.
 * @param whtype		Set this value to true if class TRECT has a constructor
 *						with four integers as left, top, width, height like Qt QRect.
 *						Set this value to false if class TRECT has a constructor
 *						with four integers as left, top, right, bottom like MFC CRect.
 * @return				A rectangle of the object window in micrometers.
 */
template<class TRECT>
TRECT FingerScanner::Information::GetSize(int objectWindow, bool whtype /* true */) const {

	Util::Variant vfr = Info.GetChild(Util::VariantId::WindowList, 0)[objectWindow];
	Util::Frame fr = vfr.ToFrame();
	if(whtype)
		return TRECT(fr.x1, fr.y1, fr.x3 - fr.x1, fr.y3 - fr.y1);
	else
		return TRECT(fr.x1, fr.y1, fr.x3, fr.y3);
}

//------------------------------------------------------------------------------

/// Returns the scanner information.
FingerScanner::Information FingerScanner::GetInfo() const {
	return PrIns::Invoker().Construct(ApiHandle, (Information*)0);
}

/** Returns the data of a previously scanned finger.
 *
 * @param position	The Imaging::FingerPosition.
 * @param mode		\ref Imaging::ImpressionType "Scan" mode.
 * @return			The Imaging::FingerImage.
 */
Imaging::FingerImage FingerScanner::GetFinger(const Imaging::FingerPosition & position,
				const Imaging::ImpressionType & mode) {
	PrIns::fpsGetImage gi = { mode, position, PrIns::ImageType::Finger, 0 };
	ApiHandle.Call(PrIns::FpsApiCallCodes::GetImage, &gi);
	return PrIns::Invoker().Construct(ApiHandle, Util::Variant(gi.Img), position, mode, (Imaging::FingerImage*)0);
}

/** Returns a set of previously scanned fingers to save.
 *
 * @param task		\ref Task::FingerTask "Filter" for fingers.
 * @return			A Processing::FingerCollection set of fingers.
 */
Processing::FingerCollection FingerScanner::GetFingerCollection(const Task::FingerTask & task) {
	return PrIns::Invoker().Construct(ApiHandle, task, (Processing::FingerCollection*)0);
}

/** Returns the last captured image.
 *
 * \note To get live images some Task::FingerTask should be started before.
 *
 * @return	The requested Imaging::RawImage.
 */
Imaging::RawImage FingerScanner::GetLiveImage(void) {
	PrIns::fpsGetImage gi = { 0, 0, PrIns::ImageType::Preview, 0 };
	ApiHandle.Call(PrIns::FpsApiCallCodes::GetImage, &gi);
	return Imaging::RawImage().FromVariant(Util::Variant(gi.Img));
}

/** Starts a task declared by the task parameter.
 *
 * \note After the task is finished the Task::TaskControl.Wait method must be called.
 *
 * @param task		The Task::FingerTask to start.
 * @return			A Task::TaskControl object.
 */
Task::TaskControl FingerScanner::StartTask(const Task::FingerTask & task) {
	if(task.task==0xf1a7 || task.task==0x9011ed) {
		struct fpsScan {
			int timeout;
			int quality;
			int mode;
			int fingerlist;
			int reqid;
		} op = { task.GetTimeout(), task.GetQuality(),
			task.task == 0xf1a7 ? Imaging::ImpressionType::Plain : Imaging::ImpressionType::Rolled,
			0, 0 };

		int fi, left = task.fingers[6] << 4, right = task.fingers[1];
		for(fi=2; fi<=5; ++fi) right |= task.fingers[fi] << (7-fi) * 4;
		for(fi=7; fi<=10; ++fi) left |= task.fingers[fi] << (12-fi) * 4;

		op.fingerlist = left | right;
		if(right==left && right) op.fingerlist |= 0xff;
		if(left) op.fingerlist |= 0x10000000;
		if(right) op.fingerlist |= 0x20000000;

		ApiHandle.Call(PrIns::FpsApiCallCodes::CaptureStart, &op);
		return PrIns::Invoker().Construct(ApiHandle, op.reqid, (Task::TaskControl*)0);
	}
	else {
		PrIns::cmProcess op = { task.task, 0 };
		ApiHandle.Call(PrIns::ApiCallCodes::StartFrvTask, &op);
		return PrIns::Invoker().Construct(ApiHandle, op.ProcessId, (Task::TaskControl*)0);
	}
}

/// Removes all the internally stored finger data from the system.
void FingerScanner::CleanUpData(void) {
	PrIns::prResetDocument rd = { 0 };
	ApiHandle.Call(PrIns::ApiCallCodes::ResetDocument, &rd);
}

} // namespace Pr22
//------------------------------------------------------------------------------
#endif //PR22_FINGERSCANNER_INCL
