#ifndef PR22_FINGERTASK_INCL
#define PR22_FINGERTASK_INCL
/******************************************************************************/
/**	\file
 *	Fingerprint scanner task.
 */
/******************************************************************************/
#define PR22_VERSION 2.2.0.20
#include "Imaging/Imaging.hpp"
#include "Processing/Processing.hpp"
#include <assert.h>
namespace Pr22 { namespace Processing { class FingerCollection; } class FingerScanner; }
//------------------------------------------------------------------------------
namespace Pr22 { namespace Task {

/// Fingerprint scanner task.
class FingerTask
{
public:

	inline FingerTask();			// Default constructor.

	/// Returns the quality requirement of the PlainScan task.
	int GetQuality() const { return quality; }

	/// Returns the timeout of the PlainScan task.
	int GetTimeout() const { return timeout; }

	// Declares a plain scan fingerprint collecting task.
	inline static FingerTask PlainScan(int quality, int timeout);
	// Appends a finger to the finger scanning list of the PlainScan task.
	inline FingerTask & Add(const Imaging::FingerPosition & finger);
	inline FingerTask & Add(const Imaging::FingerPosition & finger,
		const Processing::FingerCondition & presence);
	// Removes a finger from the finger scanning list of the PlainScan task.
	inline FingerTask & Del(const Imaging::FingerPosition & finger);

	/// Declares a live stream scanning task for viewing purposes.
	inline static FingerTask LiveStream(void) { return FingerTask(0x579ea8); }
	/// Declares a finger presence detection task.
	inline static FingerTask Detection(void) { return FingerTask(0xde7ec7); }
	// Declares a high quality live stream scanning task.
	inline static FingerTask QualityStream(void);
	// Declares a rolled scan fingerprint scanning task.
	inline static FingerTask RolledScan(const Imaging::FingerPosition & finger);

private:

	int task;									// Task Id.
	int timeout;								// Timeout of the PlainScan task.
	int quality;								// Quality requirement of the PlainScan task.
	int fingers[11];							// List of finger requirements.

	FingerTask(int t) : task(t), timeout(0), quality(0) {
		memset(fingers, 0, sizeof(fingers)); }	// Constructor for internal usage only.
	FingerTask(int quality, int timeout) : task(0xf1a7), timeout(timeout), quality(quality) {
		memset(fingers, 0, sizeof(fingers)); }	// Constructor for internal usage only.
	friend class Pr22::FingerScanner;
	friend class Pr22::Processing::FingerCollection;
};

/******************************************************************************
 * This is the end of the Finger Task interface. Now the implementation follows
 ******************************************************************************/

/// Default constructor.
FingerTask::FingerTask() : task(0), timeout(0), quality(0) {
	memset(fingers, 0, sizeof(fingers));
}

/** Appends a finger to the finger scanning list of the PlainScan task.
 *
 * @param finger	The identifier of the searched finger.
 * @return			This object.
 */
FingerTask & FingerTask::Add(const Imaging::FingerPosition & finger) {
	return Add(finger, Processing::FingerCondition::Available);
}

/** Appends a finger to the finger scanning list of the PlainScan task.
 *
 * @param finger	The identifier of a finger.
 * @param presence	The scan ability condition of the finger.
 * @return			This object.
 */
FingerTask & FingerTask::Add(const Imaging::FingerPosition & finger,
							 const Processing::FingerCondition & presence) {

	using namespace Imaging;

	switch(finger) {
	case FingerPosition::BothIndexes:
		Add(FingerPosition::LeftIndex, presence);
		return Add(FingerPosition::RightIndex, presence);

	case FingerPosition::PlainLeft4Fingers:
		Add(FingerPosition::LeftLittle, presence);
		Add(FingerPosition::LeftRing, presence);
		Add(FingerPosition::LeftMiddle, presence);
		return Add(FingerPosition::LeftIndex, presence);

	case FingerPosition::PlainRight4Fingers:
		Add(FingerPosition::RightLittle, presence);
		Add(FingerPosition::RightRing, presence);
		Add(FingerPosition::RightMiddle, presence);
		return Add(FingerPosition::RightIndex, presence);

	case FingerPosition::PlainThumbs:
		Add(FingerPosition::LeftThumb, presence);
		return Add(FingerPosition::RightThumb, presence);

	case FingerPosition::PlainLeftThumb:
		return Add(FingerPosition::LeftThumb, presence);

	case FingerPosition::PlainRightThumb:
		return Add(FingerPosition::RightThumb, presence);

	default:
		break;
	}
	assert(finger>=0 && finger<(int)gx_arraycnt(fingers));
	fingers[finger] = presence;
	return *this;
}

/** Removes a finger from the finger scanning list of the PlainScan task.
 *
 * @param finger	The identifier of the finger to remove.
 * @return			This object.
 */
FingerTask & FingerTask::Del(const Imaging::FingerPosition & finger) {
	return Add(finger, Processing::FingerCondition::NoInfo);
}

/** Declares a high quality live stream scanning task.
 *
 * Images of this stream can be analyzed but no collection processes are made.
 *
 * @return			A high quality live stream scanning task.
 */
FingerTask FingerTask::QualityStream(void) {
	return FingerTask(0x9579ea8);
}

/** Declares a plain scan fingerprint collecting task.
 *
 * @param quality	The required fingerprint quality.
 * @param timeout	Maximal time of the collection process.
 * @return			A plain scan fingerprint collecting task.
 */
FingerTask FingerTask::PlainScan(int quality, int timeout) {
	return FingerTask(quality, timeout);
}

/** Declares a rolled scan fingerprint scanning task.
 *
 * @param finger	The identifier of the requested finger.
 * @return			A rolled scan fingerprint scanning task.
 */
FingerTask FingerTask::RolledScan(const Imaging::FingerPosition & finger) {
	return FingerTask(0x9011ed).Add(finger);
}

} // namespace Task
} // namespace Pr22
//------------------------------------------------------------------------------
#endif //PR22_FINGERTASK_INCL
