/* This example shows how to get general information about the device capabilities.
 */

#include "Fps22.hpp"
#include <iostream>
#ifdef WIN32
#include <conio.h>
#endif
#ifdef QT_CORE_LIB
#include <QRect>
#else
struct QRect {
	int x, y, w, h;
	QRect() : x(0), y(0), w(0), h(0) { }
	QRect(int x, int y, int w, int h) : x(x), y(y), w(w), h(h) { }
	int width() { return w; }
	int height() { return h; }
};
#endif

namespace tutorial
{
	using namespace Pr22;

	class MainClass : public Events::FingerEventHandler
	{
		FingerprintScannerDevice *fps;

	public:

		MainClass() : fps(0) { }

		~MainClass() { delete fps; }

		//----------------------------------------------------------------------
		// Opens the first document reader device.
		int Open()
		{
			std::wcout << L"Opening a device" << std::endl;
			std::wcout << std::endl;
			fps = new FingerprintScannerDevice();

			fps->AddEventHandler(Events::Event::Connection, this);
			fps->AddEventHandler(Events::Event::DeviceUpdate, this);

			try { fps->UseDevice(0); }
			catch(const Exceptions::NoSuchDevice &)
			{
				std::wcout << L"No device found!" << std::endl;
				return 1;
			}

			std::wcout << L"The device " << fps->GetDeviceName() << L" is opened." << std::endl;
			std::wcout << std::endl;
			return 0;
		}
		//----------------------------------------------------------------------

		int Program()
		{
			//Devices can be manipulated only after opening.
			if(Open() != 0) return 1;

			std::wcout << L"SDK versions:" << std::endl;
			std::wcout << L"\tInterface: " << fps->GetVersion('A') << std::endl;
			std::wcout << L"\tSystem: " << fps->GetVersion('S') << std::endl;
			std::wcout << std::endl;

			FingerScanner::Information scannerinfo = fps->GetScanner().GetInfo();

			//Devices provide proper image quality only if they are calibrated.
			//Devices are calibrated by default. If you receive the message "not calibrated"
			//then please contact your hardware supplier.
			std::wcout << L"Calibration state of the device:" << std::endl;
			if(scannerinfo.IsCalibrated())
				std::wcout << L"\tcalibrated" << std::endl;
			else
				std::wcout << L"\tnot calibrated" << std::endl;
			std::wcout << std::endl;

			QRect frame = scannerinfo.GetSize<QRect>(0);
			std::wcout << L"Window size: " << frame.width() / 1000.0f << L" x " << frame.height() / 1000.0f << L" mm" << std::endl;
			std::wcout << std::endl;

			std::wcout << L"Scanner component versions:" << std::endl;
			std::wcout << L"\tFirmware: " << scannerinfo.GetVersion('F') << std::endl;
			std::wcout << L"\tHardware: " << scannerinfo.GetVersion('H') << std::endl;
			std::wcout << L"\tSoftware: " << scannerinfo.GetVersion('S') << std::endl;
			std::wcout << std::endl;

			std::wcout << L"Available status LEDs:" << std::endl;
			std::vector<Control::StatusLed> leds = fps->GetPeripherals().GetStatusLeds();
			for(size_t ix=0; ix<leds.size(); ++ix)
				std::wcout << L"\t" << ix << L": color " << leds[ix].GetLight().ToString() << std::endl;
			std::wcout << std::endl;

			std::wcout << L"Closing the device." << std::endl;
			fps->Close();
			return 0;
		}
		//----------------------------------------------------------------------
		// Event handlers
		//----------------------------------------------------------------------

		void OnConnection(int devno)
		{
			std::wcout << L"Connection event. Device number: " << devno << std::endl;
		}
		//----------------------------------------------------------------------

		void OnDeviceUpdate(int part)
		{
			std::wcout << L"Update event." << std::endl;
			switch(part)
			{
			case 1:
				std::wcout << L"  Reading calibration file from device." << std::endl;
				break;
			case 2:
				std::wcout << L"  Scanner firmware update." << std::endl;
				break;
			case 4:
				std::wcout << L"  RFID reader firmware update." << std::endl;
				break;
			case 5:
				std::wcout << L"  License update." << std::endl;
				break;
			}
		}
		//----------------------------------------------------------------------
	};
}

int main()
{
	try
	{
		tutorial::MainClass prog;
		prog.Program();
	}
	catch(const Pr22::Exceptions::General &e)
	{
		std::wcerr << e.what() << std::endl;
	}
#ifdef WIN32
	std::wcout << L"Press any key to exit!" << std::endl;
	_getch();
#endif
	return 0;
}
