/* This example shows how to parametrize the image scanning process.
 */

#include "Fps22.hpp"
#include <iostream>
#include <iomanip>
#ifdef WIN32
#include <conio.h>
#else
#include <unistd.h>
void Sleep(int ms) {
	sleep(ms/1000);
	usleep((ms%1000)*1000);
}
#endif

namespace tutorial
{
	using namespace Pr22;
	using namespace Pr22::Imaging;
	using namespace Pr22::Processing;
	using namespace Pr22::Task;
	using namespace Pr22::Util;

	class MainClass : public Events::FingerEventHandler
	{
		FingerprintScannerDevice *fps;
		//progress status values
		int Resolution;
		int ScanState;
		int Progress;
		int Quality[4];
		std::wstring Message;
		PresenceState Detect;

	public:

		MainClass() : fps(0), Resolution(0), ScanState(0), Progress(0), Detect(PresenceState::NoMove) {
			Quality[0] = Quality[1] = Quality[2] = Quality[3] = 0;
		}

		~MainClass() { delete fps; }

		//----------------------------------------------------------------------
		// Opens the first document reader device.
		int Open()
		{
			std::wcout << L"Opening a device" << std::endl;
			std::wcout << std::endl;
			fps = new FingerprintScannerDevice();

			fps->AddEventHandler(Events::Event::Connection, this);
			fps->AddEventHandler(Events::Event::DeviceUpdate, this);

			try { fps->UseDevice(0); }
			catch(const Exceptions::NoSuchDevice &)
			{
				std::wcout << L"No device found!" << std::endl;
				return 1;
			}

			std::wcout << L"The device " << fps->GetDeviceName() << L" is opened." << std::endl;
			std::wcout << std::endl;
			return 0;
		}
		//----------------------------------------------------------------------

		int Program()
		{
			//Devices can be manipulated only after opening.
			if(Open() != 0) return 1;

			//Subscribing to scan events
			fps->AddEventHandler(Events::Event::PreviewCaptured, this);
			fps->AddEventHandler(Events::Event::ImageScanned, this);
			fps->AddEventHandler(Events::Event::FingerPositioning, this);
			fps->AddEventHandler(Events::Event::PresenceDetection, this);

			FingerScanner &Scanner = fps->GetScanner();

			std::wcout << L"Put your left hand to the scanner." << std::endl;
			std::wcout << std::endl;

			//starting detection
			TaskControl LiveTask = Scanner.StartTask(FingerTask::Detection());

			int timeout = 100;	// in 100 milliseconds
			int i;

			for(i=0; i<timeout && Detect != PresenceState::Present; ++i)
				Sleep(100);

			//starting plain scan finger collection
			FingerTask FlatTask = FingerTask::PlainScan(800, 10000);
			FlatTask.Add(FingerPosition::PlainLeft4Fingers);
			TaskControl ScanTask = Scanner.StartTask(FlatTask);

			while(Progress < 100)
			{
				Progress = ScanTask.GetState();
				Sleep(100);
			}
			ScanTask.Wait();
			Message = L"Done";
			PrintState();

			for(i=0; i<timeout && Detect == PresenceState::Present; ++i)
				Sleep(100);

			LiveTask.Stop();

			PrintState();
			std::wcout << std::endl;
			std::wcout << std::endl;

			//saving images
			{
				Scanner.GetFinger(FingerPosition::LeftIndex, ImpressionType::Plain).
					GetImage().Save(RawImage::FileFormat::Bmp).Save("index.bmp");

				Variant nistprop = BuildNistProps();

				//excludes the middle finger from NIST saving
				FlatTask.Del(FingerPosition::LeftMiddle);

				Scanner.GetFingerCollection(FlatTask).
					Save(FingerCollection::FileFormat::Nist, nistprop).Save("mynist.nist");
			}

			//removing the finger data before a possible additional scan process
			Scanner.CleanUpData();

			std::wcout << L"Scanning processes are finished." << std::endl;
			fps->Close();
			return 0;
		}
		//----------------------------------------------------------------------

		Variant BuildNistProps()
		{
			Variant NistProp(0, Variant::List);

			//list of additional type-1 record data
			Variant Type1(1, Variant::List);
			NistProp << Type1;

			//set 1.004 to "ATP"
			Type1 << Variant(4, "ATP");

			//list of additional type-2 record data
			Variant Type2(2, Variant::List);
			NistProp << Type2;

			//set 2.045 to "Test message"
			Type2 << Variant(45, "Test message");

			return NistProp;
		}
		//----------------------------------------------------------------------

		void PrintState()
		{
			const wchar_t *States = L"|/-\\";
			const wchar_t *Prog = L"##########";

			std::wcout << L" " << States[ScanState % 4] << L" " << std::setw(3) << Resolution
				<< L" DPI [" << std::left << std::setw(10) << Prog + (10 - Progress / 10)
				<< L"] [" << std::setw(21) << Message << L"]" << std::right;
				
			for(int ix=0; ix<4; ++ix)
				std::wcout << L" Q" << ix+1 << L":" << std::setw(4) << Quality[ix];
			
			std::wcout << L"\r";
		}
		//----------------------------------------------------------------------
		// Event handlers
		//----------------------------------------------------------------------

		void OnConnection(int devno)
		{
			std::wcout << L"Connection event. Device number: " << devno << std::endl;
		}
		//----------------------------------------------------------------------

		void OnDeviceUpdate(int part)
		{
			std::wcout << L"Update event." << std::endl;
			switch(part)
			{
			case 1:
				std::wcout << L"  Reading calibration file from device." << std::endl;
				break;
			case 2:
				std::wcout << L"  Scanner firmware update." << std::endl;
				break;
			case 4:
				std::wcout << L"  RFID reader firmware update." << std::endl;
				break;
			case 5:
				std::wcout << L"  License update." << std::endl;
				break;
			}
		}
		//----------------------------------------------------------------------

		void OnPreviewCaptured(void)
		{
			Resolution = (int)(fps->GetScanner().GetLiveImage().HRes() / 39.37);
			ScanState++;
			PrintState();
		}
		//----------------------------------------------------------------------

		void OnFingerImageUpdated(const FingerPosition & position)
		{
			int pos[] = { 0, 2, 0, 1, 2, 3, 1, 3, 2, 1, 0 };
			int ix = (int)position > 10 ? 0 : (int)position;
			Quality[pos[ix]] = fps->GetScanner().GetFinger(position, ImpressionType::Plain).GetQuality();
			PrintState();
		}
		//----------------------------------------------------------------------

		void OnFingerPositioning(const FingerPositioningFailure & positioning)
		{
			Message = positioning.ToString();
			PrintState();
		}
		//----------------------------------------------------------------------

		void OnPresenceStateChanged(const PresenceState & state)
		{
			Message = state.ToString();
			PrintState();
			Detect = state;
		}
		//----------------------------------------------------------------------
	};
}

int main()
{
	try
	{
		tutorial::MainClass prog;
		prog.Program();
	}
	catch(const Pr22::Exceptions::General &e)
	{
		std::wcerr << e.what() << std::endl;
	}
#ifdef WIN32
	std::wcout << L"Press any key to exit!" << std::endl;
	_getch();
#endif
	return 0;
}
