/* This example shows how to parametrize the rolled scanning process.
 */

#include "Fps22.hpp"
#include <iostream>
#include <iomanip>
#ifdef WIN32
#include <conio.h>
#else
#include <unistd.h>
void Sleep(int ms) {
	sleep(ms/1000);
	usleep((ms%1000)*1000);
}
#endif

namespace tutorial
{
	using namespace Pr22;
	using namespace Pr22::Imaging;
	using namespace Pr22::Task;

	class MainClass : public Events::FingerEventHandler
	{
		FingerprintScannerDevice *fps;
		//progress status values
		int Resolution;
		int ScanState;
		int Progress;
		std::wstring Message;

	public:

		MainClass() : fps(0), Resolution(0), ScanState(0), Progress(0) { }

		~MainClass() { delete fps; }

		//----------------------------------------------------------------------
		// Opens the first document reader device.
		int Open()
		{
			std::wcout << L"Opening a device" << std::endl;
			std::wcout << std::endl;
			fps = new FingerprintScannerDevice();

			fps->AddEventHandler(Events::Event::Connection, this);
			fps->AddEventHandler(Events::Event::DeviceUpdate, this);

			try { fps->UseDevice(0); }
			catch(const Exceptions::NoSuchDevice &)
			{
				std::wcout << L"No device found!" << std::endl;
				return 1;
			}

			std::wcout << L"The device " << fps->GetDeviceName() << L" is opened." << std::endl;
			std::wcout << std::endl;
			return 0;
		}
		//----------------------------------------------------------------------

		int Program()
		{
			//Devices can be manipulated only after opening.
			if(Open() != 0) return 1;

			//Subscribing to scan events
			fps->AddEventHandler(Events::Event::PreviewCaptured, this);
			fps->AddEventHandler(Events::Event::ImageScanned, this);

			FingerScanner &Scanner = fps->GetScanner();

			std::wcout << L"Roll your left index finger on the scanner." << std::endl;
			std::wcout << std::endl;

			//starting rolling scan finger collection
			FingerTask RollTask = FingerTask::RolledScan(FingerPosition::LeftIndex);
			TaskControl ScanTask = Scanner.StartTask(RollTask);

			while(Progress < 100)
			{
				Progress = ScanTask.GetState();
				Sleep(100);
			}
			ScanTask.Wait();
			Message = L"Done";
			PrintState();

			std::wcout << std::endl;
			std::wcout << std::endl;

			//saving images
			{
				Scanner.GetFinger(FingerPosition::LeftIndex, ImpressionType::Rolled).
					GetImage().Save(RawImage::FileFormat::Bmp).Save("index.bmp");
			}

			//removing the finger data before a possible additional scan process
			Scanner.CleanUpData();

			std::wcout << L"Scanning processes are finished." << std::endl;
			fps->Close();
			return 0;
		}
		//----------------------------------------------------------------------

		void PrintState()
		{
			const wchar_t *States = L"|/-\\";
			const wchar_t *Prog = L"##########";

			std::wcout << L" " << States[ScanState % 4] << L" " << std::setw(3) << Resolution
				<< L" DPI [" << std::left << std::setw(10) << Prog + (10 - Progress / 10)
				<< L"] [" << std::setw(21) << Message << L"]\r";
		}
		//----------------------------------------------------------------------
		// Event handlers
		//----------------------------------------------------------------------

		void OnConnection(int devno)
		{
			std::wcout << L"Connection event. Device number: " << devno << std::endl;
		}
		//----------------------------------------------------------------------

		void OnDeviceUpdate(int part)
		{
			std::wcout << L"Update event." << std::endl;
			switch(part)
			{
			case 1:
				std::wcout << L"  Reading calibration file from device." << std::endl;
				break;
			case 2:
				std::wcout << L"  Scanner firmware update." << std::endl;
				break;
			case 4:
				std::wcout << L"  RFID reader firmware update." << std::endl;
				break;
			case 5:
				std::wcout << L"  License update." << std::endl;
				break;
			}
		}
		//----------------------------------------------------------------------

		void OnPreviewCaptured(void)
		{
			Resolution = (int)(fps->GetScanner().GetLiveImage().HRes() / 39.37);
			ScanState++;
			PrintState();
		}
		//----------------------------------------------------------------------

		void OnFingerImageUpdated(const FingerPosition & position)
		{
			fps->GetScanner().GetFinger(position, ImpressionType::Rolled).GetImage().
				Save(RawImage::FileFormat::Bmp).Save(Util::ToString(L"%02d.bmp", ScanState));
		}
		//----------------------------------------------------------------------
	};
}

int main()
{
	try
	{
		tutorial::MainClass prog;
		prog.Program();
	}
	catch(const Pr22::Exceptions::General &e)
	{
		std::wcerr << e.what() << std::endl;
	}
#ifdef WIN32
	std::wcout << L"Press any key to exit!" << std::endl;
	_getch();
#endif
	return 0;
}
