﻿/* This example shows how to get general information about the device capabilities.
 */
namespace tutorial
{
    using System.Collections.Generic;
    using Pr22;

    class MainClass
    {
        FingerprintScannerDevice fps = null;

        //----------------------------------------------------------------------
        /// <summary>
        /// Opens the first document reader device.
        /// </summary>
        /// <returns></returns>
        public int Open()
        {
            System.Console.WriteLine("Opening a device");
            System.Console.WriteLine();
            fps = new FingerprintScannerDevice();

            fps.Connection += onDeviceConnected;
            fps.DeviceUpdate += onDeviceUpdate;

            try { fps.UseDevice(0); }
            catch (Pr22.Exceptions.NoSuchDevice)
            {
                System.Console.WriteLine("No device found!");
                return 1;
            }

            System.Console.WriteLine("The device " + fps.DeviceName + " is opened.");
            System.Console.WriteLine();
            return 0;
        }
        //----------------------------------------------------------------------

        public int Program()
        {
            //Devices can be manipulated only after opening.
            if (Open() != 0) return 1;

            System.Console.WriteLine("SDK versions:");
            System.Console.WriteLine("\tAssembly: " + fps.GetType().Assembly.GetName().Version);
            System.Console.WriteLine("\tInterface: " + fps.GetVersion('A'));
            System.Console.WriteLine("\tSystem: " + fps.GetVersion('S'));
            System.Console.WriteLine();

            FingerScanner.Information scannerinfo = fps.Scanner.Info;

            //Devices provide proper image quality only if they are calibrated.
            //Devices are calibrated by default. If you receive the message "not calibrated"
            //then please contact your hardware supplier.
            System.Console.WriteLine("Calibration state of the device:");
            if (scannerinfo.IsCalibrated())
                System.Console.WriteLine("\tcalibrated");
            else
                System.Console.WriteLine("\tnot calibrated");
            System.Console.WriteLine();

            System.Drawing.Rectangle frame = scannerinfo.GetSize(0);
            System.Console.WriteLine("Window size: " + frame.Width / 1000 + " x " + frame.Height / 1000 + " mm");
            System.Console.WriteLine();

            System.Console.WriteLine("Scanner component versions:");
            System.Console.WriteLine("\tFirmware: " + scannerinfo.GetVersion('F'));
            System.Console.WriteLine("\tHardware: " + scannerinfo.GetVersion('H'));
            System.Console.WriteLine("\tSoftware: " + scannerinfo.GetVersion('S'));
            System.Console.WriteLine();

            System.Console.WriteLine("Available status LEDs:");
            List<Pr22.Control.StatusLed> leds = fps.Peripherals.StatusLeds;
            for (int i = 0; i < leds.Count; ++i)
                System.Console.WriteLine("\t" + i + ": color " + leds[i].Light);
            System.Console.WriteLine();

            System.Console.WriteLine("Closing the device.");
            fps.Close();
            return 0;
        }
        //----------------------------------------------------------------------
        // Event handlers
        //----------------------------------------------------------------------

        void onDeviceConnected(object a, Pr22.Events.ConnectionEventArgs e)
        {
            System.Console.WriteLine("Connection event. Device number: " + e.DeviceNumber);
        }
        //----------------------------------------------------------------------

        void onDeviceUpdate(object a, Pr22.Events.UpdateEventArgs e)
        {
            System.Console.WriteLine("Update event.");
            switch (e.part)
            {
                case 1:
                    System.Console.WriteLine("  Reading calibration file from device.");
                    break;
                case 2:
                    System.Console.WriteLine("  Scanner firmware update.");
                    break;
                case 4:
                    System.Console.WriteLine("  RFID reader firmware update.");
                    break;
                case 5:
                    System.Console.WriteLine("  License update.");
                    break;
            }
        }
        //----------------------------------------------------------------------

        public static int Main(string[] args)
        {
            try
            {
                MainClass prog = new MainClass();
                prog.Program();
            }
            catch (Pr22.Exceptions.General e)
            {
                System.Console.Error.WriteLine(e.Message);
            }
            System.Console.WriteLine("Press any key to exit!");
            System.Console.ReadKey(true);
            return 0;
        }
        //----------------------------------------------------------------------
    }
}
