﻿/* This example shows how to parametrize the image scanning process.
 */
namespace tutorial
{
    using System.Collections.Generic;
    using Pr22;
    using Pr22.Exceptions;
    using Pr22.Imaging;
    using Pr22.Processing;
    using Pr22.Task;
    using Pr22.Util;

    class MainClass
    {
        FingerprintScannerDevice fps = null;
        //progress status values
        int Resolution = 0;
        int ScanState = 0;
        int Progress = 0;
        int[] Quality = new int[4];
        string Message = "";
        PresenceState Detect = PresenceState.NoMove;

        //----------------------------------------------------------------------
        /// <summary>
        /// Opens the first document reader device.
        /// </summary>
        /// <returns></returns>
        public int Open()
        {
            System.Console.WriteLine("Opening a device");
            System.Console.WriteLine();
            fps = new FingerprintScannerDevice();

            fps.Connection += onDeviceConnected;
            fps.DeviceUpdate += onDeviceUpdate;

            try { fps.UseDevice(0); }
            catch (Pr22.Exceptions.NoSuchDevice)
            {
                System.Console.WriteLine("No device found!");
                return 1;
            }

            System.Console.WriteLine("The device " + fps.DeviceName + " is opened.");
            System.Console.WriteLine();
            return 0;
        }
        //----------------------------------------------------------------------

        public int Program()
        {
            //Devices can be manipulated only after opening.
            if (Open() != 0) return 1;

            //Subscribing to scan events
            fps.PreviewCaptured += PreviewCaptured;
            fps.FingerImageUpdated += FingerImageUpdated;
            fps.FingerPositioning += FingerPositioning;
            fps.PresenceStateChanged += PresenceStateChanged;

            FingerScanner Scanner = fps.Scanner;

            System.Console.WriteLine("Put your left hand to the scanner.");
            System.Console.WriteLine();

            //starting detection
            TaskControl LiveTask = Scanner.StartTask(FingerTask.Detection());

            int timeout = 100;  // in 100 milliseconds
            int i;

            for (i = 0; i < timeout && Detect != PresenceState.Present; ++i)
                System.Threading.Thread.Sleep(100);

            //starting plain scan finger collection
            FingerTask FlatTask = FingerTask.PlainScan(800, 10000);
            FlatTask.Add(FingerPosition.PlainLeft4Fingers);
            TaskControl ScanTask = Scanner.StartTask(FlatTask);

            while (Progress < 100)
            {
                Progress = ScanTask.GetState();
                System.Threading.Thread.Sleep(100);
            }
            ScanTask.Wait();
            Message = "Done";
            PrintState();

            for (i = 0; i < timeout && Detect == PresenceState.Present; ++i)
                System.Threading.Thread.Sleep(100);

            LiveTask.Stop();

            PrintState();
            System.Console.WriteLine();
            System.Console.WriteLine();

            //saving images
            {
                Scanner.GetFinger(FingerPosition.LeftIndex, ImpressionType.Plain).
                    GetImage().Save(RawImage.FileFormat.Bmp).Save("index.bmp");

                Variant nistprop = BuildNistProps();

                //excludes the middle finger from NIST saving
                FlatTask.Del(FingerPosition.LeftMiddle);

                Scanner.GetFingerCollection(FlatTask).
                        Save(FingerCollection.FileFormat.Nist, nistprop).Save("mynist.nist");
            }

            //removing the finger data before a possible additional scan process
            Scanner.CleanUpData();

            System.Console.WriteLine("Scanning processes are finished.");
            fps.Close();
            return 0;
        }
        //----------------------------------------------------------------------

        Variant BuildNistProps()
        {
            Variant NistProp = new Variant(0, Variant.ListT.List);

            //list of additional type-1 record data
            Variant Type1 = new Variant(1, Variant.ListT.List);
            NistProp.List += Type1;

            //set 1.004 to "ATP"
            Type1.List += new Variant(4, "ATP");

            //list of additional type-2 record data
            Variant Type2 = new Variant(2, Variant.ListT.List);
            NistProp.List += Type2;

            //set 2.045 to "Test message"
            Type2.List += new Variant(45, "Test message");

            return NistProp;
        }
        //----------------------------------------------------------------------

        void PrintState()
        {
            string States = "|/-\\";
            string Prog = "##########";

            System.Console.Write(" {0} {1,3} DPI [{2,-10}] [{3,-21}] Q1:{4,4} Q2:{5,4} Q3:{6,4} Q4:{7,4}\r",
                States[ScanState % 4], Resolution, Prog.Substring(10 - Progress / 10),
                Message, Quality[0], Quality[1], Quality[2], Quality[3]);
        }
        //----------------------------------------------------------------------
        // Event handlers
        //----------------------------------------------------------------------

        void onDeviceConnected(object a, Pr22.Events.ConnectionEventArgs e)
        {
            System.Console.WriteLine("Connection event. Device number: " + e.DeviceNumber);
        }
        //----------------------------------------------------------------------

        void onDeviceUpdate(object a, Pr22.Events.UpdateEventArgs e)
        {
            System.Console.WriteLine("Update event.");
            switch (e.part)
            {
                case 1:
                    System.Console.WriteLine("  Reading calibration file from device.");
                    break;
                case 2:
                    System.Console.WriteLine("  Scanner firmware update.");
                    break;
                case 4:
                    System.Console.WriteLine("  RFID reader firmware update.");
                    break;
                case 5:
                    System.Console.WriteLine("  License update.");
                    break;
            }
        }
        //----------------------------------------------------------------------

        void PreviewCaptured(object a, Pr22.Events.PreviewEventArgs e)
        {
            Resolution = (int)(fps.Scanner.GetLiveImage().HRes / 39.37);
            ScanState++;
            PrintState();
        }
        //----------------------------------------------------------------------

        void FingerImageUpdated(object a, Pr22.Events.FingerUpdateEventArgs e)
        {
            int[] pos ={ 0, 2, 0, 1, 2, 3, 1, 3, 2, 1, 0 };
            int ix = (int)e.position > 10 ? 0 : (int)e.position;
            Quality[pos[ix]] = fps.Scanner.GetFinger(e.position, ImpressionType.Plain).GetQuality();
            PrintState();
        }
        //----------------------------------------------------------------------

        void FingerPositioning(object a, Pr22.Events.FingerEventArgs e)
        {
            Message = e.fingerFailureMask.ToString();
            PrintState();
        }
        //----------------------------------------------------------------------

        void PresenceStateChanged(object a, Pr22.Events.DetectionEventArgs e)
        {
            Message = e.State.ToString();
            PrintState();
            Detect = e.State;
        }
        //----------------------------------------------------------------------

        public static int Main(string[] args)
        {
            try
            {
                MainClass prog = new MainClass();
                prog.Program();
            }
            catch (Pr22.Exceptions.General e)
            {
                System.Console.Error.WriteLine(e.Message);
            }
            System.Console.WriteLine("Press any key to exit!");
            System.Console.ReadKey(true);
            return 0;
        }
        //----------------------------------------------------------------------
    }
}
