﻿/* This example shows how to parametrize the rolled scanning process.
 */
namespace tutorial
{
    using System.Collections.Generic;
    using Pr22;
    using Pr22.Exceptions;
    using Pr22.Imaging;
    using Pr22.Processing;
    using Pr22.Task;
    using Pr22.Util;

    class MainClass
    {
        FingerprintScannerDevice fps = null;
        //progress status values
        int Resolution = 0;
        int ScanState = 0;
        int Progress = 0;
        string Message = "";

        //----------------------------------------------------------------------
        /// <summary>
        /// Opens the first document reader device.
        /// </summary>
        /// <returns></returns>
        public int Open()
        {
            System.Console.WriteLine("Opening a device");
            System.Console.WriteLine();
            fps = new FingerprintScannerDevice();

            fps.Connection += onDeviceConnected;
            fps.DeviceUpdate += onDeviceUpdate;

            try { fps.UseDevice(0); }
            catch (Pr22.Exceptions.NoSuchDevice)
            {
                System.Console.WriteLine("No device found!");
                return 1;
            }

            System.Console.WriteLine("The device " + fps.DeviceName + " is opened.");
            System.Console.WriteLine();
            return 0;
        }
        //----------------------------------------------------------------------

        public int Program()
        {
            //Devices can be manipulated only after opening.
            if (Open() != 0) return 1;

            //Subscribing to scan events
            fps.PreviewCaptured += PreviewCaptured;
            fps.FingerImageUpdated += FingerImageUpdated;

            FingerScanner Scanner = fps.Scanner;

            System.Console.WriteLine("Roll your left index finger on the scanner.");
            System.Console.WriteLine();

            //starting rolling scan finger collection
            FingerTask RollTask = FingerTask.RolledScan(FingerPosition.LeftIndex);
            TaskControl ScanTask = Scanner.StartTask(RollTask);

            while (Progress < 100)
            {
                Progress = ScanTask.GetState();
                System.Threading.Thread.Sleep(100);
            }
            ScanTask.Wait();
            Message = "Done";
            PrintState();

            System.Console.WriteLine();
            System.Console.WriteLine();

            //saving images
            {
                Scanner.GetFinger(FingerPosition.LeftIndex, ImpressionType.Rolled).
                    GetImage().Save(RawImage.FileFormat.Bmp).Save("index.bmp");
            }

            //removing the finger data before a possible additional scan process
            Scanner.CleanUpData();

            System.Console.WriteLine("Scanning processes are finished.");
            fps.Close();
            return 0;
        }
        //----------------------------------------------------------------------

        void PrintState()
        {
            string States = "|/-\\";
            string Prog = "##########";

            System.Console.Write(" {0} {1,3} DPI [{2,-10}] [{3,-21}]\r",
                States[ScanState % 4], Resolution, Prog.Substring(10 - Progress / 10), Message);
        }
        //----------------------------------------------------------------------
        // Event handlers
        //----------------------------------------------------------------------

        void onDeviceConnected(object a, Pr22.Events.ConnectionEventArgs e)
        {
            System.Console.WriteLine("Connection event. Device number: " + e.DeviceNumber);
        }
        //----------------------------------------------------------------------

        void onDeviceUpdate(object a, Pr22.Events.UpdateEventArgs e)
        {
            System.Console.WriteLine("Update event.");
            switch (e.part)
            {
                case 1:
                    System.Console.WriteLine("  Reading calibration file from device.");
                    break;
                case 2:
                    System.Console.WriteLine("  Scanner firmware update.");
                    break;
                case 4:
                    System.Console.WriteLine("  RFID reader firmware update.");
                    break;
                case 5:
                    System.Console.WriteLine("  License update.");
                    break;
            }
        }
        //----------------------------------------------------------------------

        void PreviewCaptured(object a, Pr22.Events.PreviewEventArgs e)
        {
            Resolution = (int)(fps.Scanner.GetLiveImage().HRes / 39.37);
            ScanState++;
            PrintState();
        }
        //----------------------------------------------------------------------

        void FingerImageUpdated(object a, Pr22.Events.FingerUpdateEventArgs e)
        {
            fps.Scanner.GetFinger(e.position, ImpressionType.Rolled).GetImage().
                Save(RawImage.FileFormat.Bmp).Save(ScanState.ToString("d2") + ".bmp");
        }
        //----------------------------------------------------------------------

        public static int Main(string[] args)
        {
            try
            {
                MainClass prog = new MainClass();
                prog.Program();
            }
            catch (Pr22.Exceptions.General e)
            {
                System.Console.Error.WriteLine(e.Message);
            }
            System.Console.WriteLine("Press any key to exit!");
            System.Console.ReadKey(true);
            return 0;
        }
        //----------------------------------------------------------------------
    }
}
