﻿' This example shows how to parametrize the image scanning process.

Option Explicit On
Imports Microsoft.VisualBasic
Imports System.Collections.Generic
Imports Pr22
Imports Pr22.Exceptions
Imports Pr22.Imaging
Imports Pr22.Processing
Imports Pr22.Task
Imports Pr22.Util
Namespace tutorial

    Class MainClass

        Private fps As FingerprintScannerDevice = Nothing
        'progress status values
        Private Resolution As Integer = 0
        Private ScanState As Integer = 0
        Private Progress As Integer = 0
        Private Quality As Integer() = New Integer(3) {}
        Private Message As String = ""
        Private Detect As PresenceState = PresenceState.NoMove

        '----------------------------------------------------------------------
        ''' <summary>
        ''' Opens the first document reader device.
        ''' </summary>
        ''' <returns></returns>
        Public Function Open() As Integer

            System.Console.WriteLine("Opening a device")
            System.Console.WriteLine()
            fps = New FingerprintScannerDevice()

            AddHandler fps.Connection, AddressOf onDeviceConnected
            AddHandler fps.DeviceUpdate, AddressOf onDeviceUpdate

            Try
                fps.UseDevice(0)
            Catch ex As Pr22.Exceptions.NoSuchDevice
                System.Console.WriteLine("No device found!")
                Return 1
            End Try

            System.Console.WriteLine("The device " + fps.DeviceName + " is opened.")
            System.Console.WriteLine()
            Return 0
        End Function
        '----------------------------------------------------------------------

        Public Function Program() As Integer

            'Devices can be manipulated only after opening.
            If Open() <> 0 Then Return 1

            'Subscribing to scan events
            AddHandler fps.PreviewCaptured, AddressOf PreviewCaptured
            AddHandler fps.FingerImageUpdated, AddressOf FingerImageUpdated
            AddHandler fps.FingerPositioning, AddressOf FingerPositioning
            AddHandler fps.PresenceStateChanged, AddressOf PresenceStateChanged

            Dim Scanner As FingerScanner = fps.Scanner

            System.Console.WriteLine("Put your left hand to the scanner.")
            System.Console.WriteLine()

            'starting detection
            Dim LiveTask As TaskControl = Scanner.StartTask(FingerTask.Detection())

            Dim timeout As Integer = 100        ' in 100 milliseconds
            Dim i As Integer

            i = 0
            While i < timeout AndAlso Detect <> PresenceState.Present
                System.Threading.Thread.Sleep(100) : i += 1
            End While

            'starting plain scan finger collection
            Dim FlatTask As FingerTask = FingerTask.PlainScan(800, 10000)
            FlatTask.Add(FingerPosition.PlainLeft4Fingers)
            Dim ScanTask As TaskControl = Scanner.StartTask(FlatTask)

            While Progress < 100
                Progress = ScanTask.GetState()
                System.Threading.Thread.Sleep(100)
            End While
            ScanTask.Wait()
            Message = "Done"
            PrintState()

            i = 0
            While i < timeout AndAlso Detect = PresenceState.Present
                System.Threading.Thread.Sleep(100) : i += 1
            End While

            LiveTask.Stop()

            PrintState()
            System.Console.WriteLine()
            System.Console.WriteLine()

            'saving images
            Scanner.GetFinger(FingerPosition.LeftIndex, ImpressionType.Plain). _
                GetImage().Save(RawImage.FileFormat.Bmp).Save("index.bmp")

            Dim nistprop As Pr22.Util.Variant = BuildNistProps()

            'excludes the middle finger from NIST saving
            FlatTask.Del(FingerPosition.LeftMiddle)

            Scanner.GetFingerCollection(FlatTask). _
                Save(FingerCollection.FileFormat.Nist, nistprop).Save("mynist.nist")

            'removing the finger data before a possible additional scan process
            Scanner.CleanUpData()

            System.Console.WriteLine("Scanning processes are finished.")
            fps.Close()
            Return 0
        End Function
        '----------------------------------------------------------------------

        Private Function BuildNistProps() As Pr22.Util.Variant

            Dim NistProp As New Pr22.Util.Variant(0, Pr22.Util.Variant.ListT.List)

            'list of additional type-1 record data
            Dim Type1 As New Pr22.Util.Variant(1, Pr22.Util.Variant.ListT.List)
            NistProp.List += Type1

            'set 1.004 to "ATP"
            Type1.List += New Pr22.Util.Variant(4, "ATP")

            'list of additional type-2 record data
            Dim Type2 As New Pr22.Util.Variant(2, Pr22.Util.Variant.ListT.List)
            NistProp.List += Type2

            'set 2.045 to "Test message"
            Type2.List += New Pr22.Util.Variant(45, "Test message")

            Return NistProp
        End Function
        '----------------------------------------------------------------------

        Private Sub PrintState()

            Dim States As String = "|/-\"
            Dim Prog As String = "##########"

            System.Console.Write(" {0} {1,3} DPI [{2,-10}] [{3,-21}] Q1:{4,4} Q2:{5,4} Q3:{6,4} Q4:{7,4}" & vbCr, _
                States(ScanState Mod 4), Resolution, Prog.Substring(10 - Progress \ 10), _
                Message, Quality(0), Quality(1), Quality(2), Quality(3))
        End Sub
        '----------------------------------------------------------------------
        ' Event handlers
        '----------------------------------------------------------------------

        Private Sub onDeviceConnected(ByVal a As Object, ByVal e As Pr22.Events.ConnectionEventArgs)

            System.Console.WriteLine("Connection event. Device number: " & e.DeviceNumber)
        End Sub
        '----------------------------------------------------------------------

        Private Sub onDeviceUpdate(ByVal a As Object, ByVal e As Pr22.Events.UpdateEventArgs)

            System.Console.WriteLine("Update event.")
            Select Case e.part
                Case 1
                    System.Console.WriteLine("  Reading calibration file from device.")
                Case 2
                    System.Console.WriteLine("  Scanner firmware update.")
                Case 4
                    System.Console.WriteLine("  RFID reader firmware update.")
                Case 5
                    System.Console.WriteLine("  License update.")
            End Select
        End Sub
        '----------------------------------------------------------------------

        Private Sub PreviewCaptured(ByVal a As Object, ByVal e As Pr22.Events.PreviewEventArgs)

            Resolution = CInt((fps.Scanner.GetLiveImage().HRes / 39.37))
            ScanState += 1
            PrintState()
        End Sub
        '----------------------------------------------------------------------

        Private Sub FingerImageUpdated(ByVal a As Object, ByVal e As Pr22.Events.FingerUpdateEventArgs)

            Dim pos As Integer() = {0, 2, 0, 1, 2, 3, 1, 3, 2, 1, 0}
            Dim ix As Integer = CInt(IIf(CInt(e.position) > 10, 0, CInt(e.position)))
            Quality(pos(ix)) = fps.Scanner.GetFinger(e.position, ImpressionType.Plain).GetQuality()
            PrintState()
        End Sub
        '----------------------------------------------------------------------

        Private Sub FingerPositioning(ByVal a As Object, ByVal e As Pr22.Events.FingerEventArgs)

            Message = e.fingerFailureMask.ToString()
            PrintState()
        End Sub
        '----------------------------------------------------------------------

        Private Sub PresenceStateChanged(ByVal a As Object, ByVal e As Pr22.Events.DetectionEventArgs)

            Message = e.State.ToString()
            PrintState()
            Detect = e.State
        End Sub
        '----------------------------------------------------------------------

        Public Shared Function Main(ByVal args As String()) As Integer

            Try
                Dim prog As New MainClass()
                prog.Program()
            Catch e As Pr22.Exceptions.General
                System.Console.Error.WriteLine(e.Message)
            End Try
            System.Console.WriteLine("Press any key to exit!")
            System.Console.ReadKey(True)
            Return 0
        End Function
        '----------------------------------------------------------------------
    End Class
End Namespace
